@@ -138,20 +138,22 @@ gda_report_engine_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		if (type == 0) {
-			static GTypeInfo info = {
-				sizeof (GdaReportEngineClass),
-				(GBaseInitFunc) NULL,
-				(GBaseFinalizeFunc) NULL,
-				(GClassInitFunc) gda_report_engine_class_init,
-				NULL, NULL,
-				sizeof (GdaReportEngine),
-				0,
-				(GInstanceInitFunc) gda_report_engine_init
-			};
-			
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GTypeInfo info = {
+			sizeof (GdaReportEngineClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) gda_report_engine_class_init,
+			NULL, NULL,
+			sizeof (GdaReportEngine),
+			0,
+			(GInstanceInitFunc) gda_report_engine_init
+		};
+		
+		g_static_mutex_lock (&registering);
+		if (type == 0)
 			type = g_type_register_static (G_TYPE_OBJECT, "GdaReportEngine", &info, 0);
-		}
+		g_static_mutex_unlock (&registering);
 	}
 
 	return type;
@@ -865,7 +867,7 @@ command_gda_report_if (GdaReportEngine *engine, xmlNodePtr node, GSList **create
 	prop = xmlGetProp (node, BAD_CAST "expr");
 	if (!prop) {
 		g_set_error (error, 0, 0,
-			     _("No expression speficied")); 
+			     _("No expression specified")); 
 		return FALSE;
 	}
 
@@ -990,19 +992,24 @@ evaluate_expression (GdaReportEngine *engine, RunContext *context, const gchar *
 	GdaDataModel *model;
 	GValue *retval;
 	gchar *sql;
+	static GStaticMutex init_mutex = G_STATIC_MUTEX_INIT;
 	GdaConnection *vcnc = NULL;
 	GdaSqlParser *parser;
 
 	/* create a virtual connection to execute the expression, if it's the first time */
+	g_static_mutex_lock (&init_mutex);
 	if (!vcnc) {
 		static GdaVirtualProvider *provider = NULL;
 
 		if (!provider)
 			provider = gda_vprovider_data_model_new ();
 		vcnc = gda_virtual_connection_open (provider, error);
-		if (! vcnc) 
+		if (! vcnc) {
+			g_static_mutex_unlock (&init_mutex);
 			return NULL;
+		}
 	}
+	g_static_mutex_unlock (&init_mutex);
 
 	/* parser */
 	parser = g_object_get_data (G_OBJECT (context->cnc), "__gda_parser");
@@ -1055,7 +1062,7 @@ evaluate_expression (GdaReportEngine *engine, RunContext *context, const gchar *
 static guint
 gtype_hash (gconstpointer key)
 {
-        return (guint) key;
+        return GPOINTER_TO_UINT (key);
 }
 
 static gboolean
