@@ -40,7 +40,8 @@ gda_blob_op_get_type (void)
 {
         static GType type = 0;
 
-        if (G_UNLIKELY (type == 0)) {
+	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
                 static const GTypeInfo info = {
                         sizeof (GdaBlobOpClass),
                         (GBaseInitFunc) NULL,
@@ -53,7 +54,10 @@ gda_blob_op_get_type (void)
                         (GInstanceInitFunc) gda_blob_op_init
                 };
 
-                type = g_type_register_static (PARENT_TYPE, "GdaBlobOp", &info, G_TYPE_FLAG_ABSTRACT);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (PARENT_TYPE, "GdaBlobOp", &info, G_TYPE_FLAG_ABSTRACT);
+		g_static_mutex_unlock (&registering);
         }
         return type;
 }
