@@ -181,6 +181,7 @@ gda_column_get_type (void)
 	static GType type = 0;
 	
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaColumnClass),
 			(GBaseInitFunc) NULL,
@@ -192,7 +193,10 @@ gda_column_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_column_init
 		};
-		type = g_type_register_static (PARENT_TYPE, "GdaColumn", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (PARENT_TYPE, "GdaColumn", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	
 	return type;
@@ -756,6 +760,6 @@ gda_column_set_default_value (GdaColumn *column, const GValue *default_value)
 	g_return_if_fail (default_value != NULL);
 
 	if (column->priv->default_value)
-		g_free (column->priv->default_value);
+		gda_value_free (column->priv->default_value);
 	column->priv->default_value = gda_value_copy ( (GValue*)default_value);
 }
