@@ -20,6 +20,7 @@
 
 #include "gda-data-handler.h"
 
+static GStaticRecMutex init_mutex = G_STATIC_REC_MUTEX_INIT;
 static void gda_data_handler_iface_init (gpointer g_class);
 
 GType
@@ -40,8 +41,12 @@ gda_data_handler_get_type (void)
 			(GInstanceInitFunc) NULL
 		};
 		
-		type = g_type_register_static (G_TYPE_INTERFACE, "GdaDataHandler", &info, 0);
-                g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
+		g_static_rec_mutex_lock (&init_mutex);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_INTERFACE, "GdaDataHandler", &info, 0);
+			g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
+		}
+		g_static_rec_mutex_unlock (&init_mutex);
 	}
 	return type;
 }
@@ -52,9 +57,11 @@ gda_data_handler_iface_init (gpointer g_class)
 {
 	static gboolean initialized = FALSE;
 
+	g_static_rec_mutex_lock (&init_mutex);
 	if (! initialized) {
 		initialized = TRUE;
 	}
+	g_static_rec_mutex_unlock (&init_mutex);
 }
 
 /**
