@@ -101,6 +101,7 @@ gda_data_model_iter_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelIterClass),
 			(GBaseInitFunc) NULL,
@@ -114,7 +115,10 @@ gda_data_model_iter_get_type (void)
 		};
 
 		
-		type = g_type_register_static (GDA_TYPE_SET, "GdaDataModelIter", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (GDA_TYPE_SET, "GdaDataModelIter", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -255,7 +259,7 @@ holder_changed_cb (GdaSet *paramlist, GdaHolder *param)
 	GdaDataModelIter *iter;
 	gint col;
 
-	iter = GDA_DATA_MODEL_ITER (paramlist);
+	iter = (GdaDataModelIter *) paramlist;
 	if (!iter->priv->keep_param_changes && (iter->priv->row >= 0)) {
 		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [0]);
 		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [1]);
@@ -264,7 +268,7 @@ holder_changed_cb (GdaSet *paramlist, GdaHolder *param)
 		col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
 		g_return_if_fail (col >= 0);
 		
-		if (! gda_data_model_set_value_at (GDA_DATA_MODEL (iter->priv->data_model), 
+		if (! gda_data_model_set_value_at ((GdaDataModel *) iter->priv->data_model, 
 						   col, iter->priv->row, gda_holder_get_value (param), NULL)) {
 			/* writing to the model failed, revert back the change to parameter */
 			iter->priv->keep_param_changes = TRUE;
