@@ -29,8 +29,10 @@
 #include <libgda/gda-data-model-extra.h>
 #include <libgda/gda-data-model-iter.h>
 #include <libgda/gda-statement.h>
+#include <libgda/gda-meta-struct.h>
 #include <libgda/gda-set.h>
 #include <libgda/gda-holder.h>
+#include <libgda/gda-util.h>
 #include <sql-parser/gda-sql-statement.h>
 
 struct _GdaDataModelQueryPrivate {
@@ -143,6 +145,7 @@ gda_data_model_query_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelQueryClass),
 			(GBaseInitFunc) NULL,
@@ -161,8 +164,12 @@ gda_data_model_query_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelQuery", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelQuery", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -425,12 +432,9 @@ gda_data_model_query_set_property (GObject *object,
 						     "expect some problems with the GdaDataModelQuery"));
 					model->priv->params[qindex] = NULL;
 				}
+
 				if (qindex == SEL_QUERY) {
 					/* SELECT statement */
-
-					/* expand any target.* field => add a gda_sql_statement_select_expand_all () method */
-					TO_IMPLEMENT;
-
 					if (model->priv->params[qindex])
 						g_signal_connect (model->priv->params[qindex], "holder_changed",
 								  G_CALLBACK (holder_changed_cb), model);
@@ -473,8 +477,6 @@ gda_data_model_query_set_property (GObject *object,
 										gda_holder_set_default_value (holder, value);
 										gda_value_free (value);
 									}
-									g_object_set ((GObject*) holder, "id",
-										      gda_column_get_name (col), NULL);
 								}
 							}
 							else {
@@ -667,7 +669,7 @@ gda_data_model_query_refresh (GdaDataModelQuery *model, GError **error)
 	}
 	if (! gda_connection_is_opened (model->priv->cnc)) {
 		g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR, GDA_DATA_MODEL_QUERY_CONNECTION_ERROR,
-			     _("Specified is closed"));
+			     _("Specified connection is closed"));
 		return FALSE;
 	}
 
@@ -777,8 +779,6 @@ gda_data_model_query_get_n_columns (GdaDataModel *model)
 static void
 create_columns (GdaDataModelQuery *model)
 {
-	GSList *fields = NULL;
-
 	if (model->priv->columns)
 		return;
 	if (! model->priv->statements[SEL_QUERY])
@@ -788,30 +788,30 @@ create_columns (GdaDataModelQuery *model)
 	 * statement's GdaSqlSelectFields and if a name and GType can be determined, then use them, otherwise
 	 * use "column%d" and G_TYPE_STRING, and let the user modify that directly using gda_data_model_describe () 
 	 */
-	TO_IMPLEMENT;
-	if (0) {
-		GSList *list;
-		gboolean allok = TRUE;
-		
-		for (list = fields; list; list = list->next) {
-		}
-		if (! allok) 
-			return;
+	GdaSqlStatement *sqlst;
+	g_object_get (G_OBJECT (model->priv->statements[SEL_QUERY]), "structure", &sqlst, NULL);
 
-		list = fields;
-		while (list) {
+	if (gda_sql_statement_normalize (sqlst, model->priv->cnc, NULL)) {
+		GdaSqlStatementSelect *selst = (GdaSqlStatementSelect*) sqlst->contents;
+		GSList *list;
+		
+		for (list = selst->expr_list; list; list = list->next) {
+			GdaSqlSelectField *field = (GdaSqlSelectField*) list->data;
 			GdaColumn *col;
 
 			col = gda_column_new ();
-			gda_column_set_name (col, "col");
-			gda_column_set_title (col, "col");
-			gda_column_set_g_type (col, G_TYPE_STRING);
+			if (field->validity_meta_table_column) {
+				gda_column_set_name (col, field->validity_meta_table_column->column_name);
+				gda_column_set_title (col, field->validity_meta_table_column->column_name);
+				gda_column_set_g_type (col, field->validity_meta_table_column->gtype);
+			}
+			else {
+				gda_column_set_name (col, "col");
+				gda_column_set_title (col, "col");
+				gda_column_set_g_type (col, G_TYPE_STRING);
+			}
 			model->priv->columns = g_slist_append (model->priv->columns, col);
-			
-			list = g_slist_next (list);
 		}
-
-		g_slist_free (fields);
 	}
 	else {
 		if (model->priv->data) {
@@ -835,6 +835,7 @@ create_columns (GdaDataModelQuery *model)
 			model->priv->emit_reset = TRUE;
 		}
 	}
+	gda_sql_statement_free (sqlst);
 
 	if (model->priv->columns && model->priv->emit_reset) {
 		model->priv->emit_reset = FALSE;
@@ -1000,7 +1001,11 @@ gda_data_model_query_create_iter (GdaDataModel *model)
 	iter = (GdaDataModelIter *) g_object_new (GDA_TYPE_DATA_MODEL_ITER, 
 						  "data_model", model, NULL);
 	/* set the "__gda_entry_plugin" property for all the parameters depending on the SELECT query field */
-	TO_IMPLEMENT;
+	static gboolean warned = FALSE;
+	if (!warned) {
+		warned = TRUE;
+		TO_IMPLEMENT;
+	}
 	/*
 	if (gda_query_is_select_query (GDA_DATA_MODEL_QUERY (model)->priv->statements[SEL_QUERY])) {
 		GSList *list, *fields;
@@ -1076,7 +1081,7 @@ run_modify_query (GdaDataModelQuery *selmodel, gint query_type, GError **error)
 	}
 	if (! gda_connection_is_opened (selmodel->priv->cnc)) {
 		g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR, GDA_DATA_MODEL_QUERY_CONNECTION_ERROR,
-			     _("Specified is closed"));
+			     _("Specified connection is closed"));
 		return FALSE;
 	}
 
@@ -1474,12 +1479,15 @@ gda_data_model_query_send_hint (GdaDataModel *model, GdaDataModelHint hint, cons
  * If @target is %NULL, then an error will be returned if @model's SELECT query has more than
  * one target.
  *
- * Returns: TRUE if the INSERT, DELETE and UPDATE statements have been computed.
+ * Returns: TRUE at least one of the INSERT, DELETE and UPDATE statements has been computed
  */
 gboolean
 gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, const gchar *target, 
 						   GdaDataModelQueryOptions options, GError **error)
 {
+	gint i;
+	gboolean require_pk = TRUE;
+	GdaStatement *ins, *upd, *del;
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_QUERY (model), FALSE);
 	g_return_val_if_fail (model->priv, FALSE);
 
@@ -1490,6 +1498,23 @@ gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, con
 		return FALSE;
 	}
 
-	TO_IMPLEMENT;
+	for (i = SEL_QUERY + 1; i <= DEL_QUERY; i++) {
+		if (model->priv->statements[i])
+			forget_statement (model, model->priv->statements[i]);
+	}
+	
+	if (options & GDA_DATA_MODEL_QUERY_OPTION_USE_ALL_FIELDS_IF_NO_PK)
+		require_pk = FALSE;
+	if (gda_compute_dml_statements (model->priv->cnc, model->priv->statements[SEL_QUERY], require_pk, 
+					&ins, &upd, &del, error)) {
+		
+		g_object_set (G_OBJECT (model), "insert_query", ins, 
+			      "update-query", upd, "delete-query", del, NULL);
+		if (ins) g_object_unref (ins);
+		if (upd) g_object_unref (upd);
+		if (del) g_object_unref (del);
+		return TRUE;
+	}
+
 	return FALSE;
 }
