@@ -176,6 +176,7 @@ gda_data_model_row_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static GTypeInfo info = {
 			sizeof (GdaDataModelRowClass),
 			(GBaseInitFunc) NULL,
@@ -193,8 +194,12 @@ gda_data_model_row_get_type (void)
 			NULL
 		};
 		
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelRow", &info, G_TYPE_FLAG_ABSTRACT);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelRow", &info, G_TYPE_FLAG_ABSTRACT);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 
 	return type;
@@ -258,7 +263,7 @@ gda_data_model_row_describe_column (GdaDataModel *model, gint col)
 	GdaColumn *column;
 
 	if (col >= gda_data_model_get_n_columns (model)) {
-		g_warning ("Column %d out of range 0 - %d", col, gda_data_model_get_n_columns (model) - 1);
+		g_warning ("Column %d out of range (0-%d)", col, gda_data_model_get_n_columns (model) - 1);
 		return NULL;
 	}
 
