@@ -40,9 +40,7 @@
 #endif
 #include "csv.h"
 
-#define PARENT_TYPE G_TYPE_OBJECT
-#define CLASS(model) (GDA_DATA_MODEL_CLASS (G_OBJECT_GET_CLASS (model)))
-
+static GStaticRecMutex init_mutex = G_STATIC_REC_MUTEX_INIT;
 static void gda_data_model_class_init (gpointer g_class);
 
 /* signals */
@@ -75,8 +73,12 @@ gda_data_model_get_type (void)
 			(GInstanceInitFunc) NULL
 		};
 		
-		type = g_type_register_static (G_TYPE_INTERFACE, "GdaDataModel", &info, 0);
-		g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
+		g_static_rec_mutex_lock (&init_mutex);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_INTERFACE, "GdaDataModel", &info, 0);
+			g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
+		}
+		g_static_rec_mutex_unlock (&init_mutex);
 	}
 	return type;
 }
@@ -86,6 +88,7 @@ gda_data_model_class_init (gpointer g_class)
 {
 	static gboolean initialized = FALSE;
 
+	g_static_rec_mutex_lock (&init_mutex);
 	if (! initialized) {
 		gda_data_model_signals[CHANGED] =
 			g_signal_new ("changed",
@@ -130,6 +133,7 @@ gda_data_model_class_init (gpointer g_class)
 
 		initialized = TRUE;
 	}
+	g_static_rec_mutex_unlock (&init_mutex);
 }
 
 static gboolean
@@ -752,7 +756,7 @@ gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *
 	g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
 	g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
 	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
-		gda_holder_set_value (GDA_HOLDER (list->data), 
+		gda_holder_set_value ((GdaHolder*) list->data, 
 				      gda_data_model_get_value_at (model, col, row));
 
 		set_param_attributes ((GdaHolder*) list->data, 
@@ -830,7 +834,7 @@ gda_data_model_move_iter_next_default (GdaDataModel *model, GdaDataModelIter *it
 	g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
 	g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
 	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
-		gda_holder_set_value (GDA_HOLDER (list->data), 
+		gda_holder_set_value ((GdaHolder *) list->data, 
 				      gda_data_model_get_value_at (model, col, row));
 		set_param_attributes ((GdaHolder *) list->data, 
 				      gda_data_model_get_attributes_at (model, col, row));
@@ -893,7 +897,7 @@ gda_data_model_move_iter_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
 	g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
 	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
-		gda_holder_set_value (GDA_HOLDER (list->data), 
+		gda_holder_set_value ((GdaHolder*) list->data, 
 				      gda_data_model_get_value_at (model, col, row));
 		set_param_attributes ((GdaHolder*) list->data,
 				      gda_data_model_get_attributes_at (model, col, row));
@@ -1124,7 +1128,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 				if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING))
 					name = g_value_get_string ((GValue *) value);
 				else
-					g_warning (_("The 'NAME' parameter must hold a string value, ignored."));
+					g_warning (_("The '%s' parameter must hold a string value, ignored."), "NAME");
 			}
 		}
 
@@ -1163,7 +1167,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 						sep = *str;
 				}
 				else
-					g_warning (_("The 'SEPARATOR' parameter must hold a string value, ignored."));
+					g_warning (_("The '%s' parameter must hold a string value, ignored."), "SEPARATOR");
 			}
 			holder = gda_set_get_holder (options, "QUOTE");
 			if (holder) {
@@ -1177,7 +1181,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 						quote = *str;
 				}
 				else 
-					g_warning (_("The 'QUOTE' parameter must hold a string value, ignored."));
+					g_warning (_("The '%s' parameter must hold a string value, ignored."), "QUOTE");
 			}
 			holder = gda_set_get_holder (options, "FIELD_QUOTE");
 			if (holder) {
@@ -1186,7 +1190,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 				if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 					field_quote = g_value_get_boolean ((GValue *) value);
 				else 
-					g_warning (_("The 'FIELD_QUOTE' parameter must hold a boolean value, ignored."));
+					g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "FIELD_QUOTE");
 			}
 		}
 
@@ -1269,7 +1273,7 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 			if (value && (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN))
 				overwrite = g_value_get_boolean ((GValue *) value);
 			else
-				g_warning (_("The 'OVERWRITE' parameter must hold a boolean value, ignored."));
+				g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "OVERWRITE");
 		}
 	}
 
@@ -1282,12 +1286,10 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 		}
 	}
 	
-	if (! gda_file_save (file, body, strlen (body))) {
-		g_set_error (error, 0, 0, _("Could not save file %s"), file);
+	if (! g_file_set_contents (file, body, -1, error)) {
 		g_free (body);
 		return FALSE;
 	}
-
 	g_free (body);
 	return TRUE;
 }
@@ -1480,8 +1482,7 @@ find_column_from_id (GdaDataModel *model, const gchar *colid, gint *pos)
 			column = NULL;
 		else
 			*pos = c;
-		if(id != NULL)
-			g_free(id);
+		g_free (id);
 	}
 
 	/* if no column has been found, assumr @colid is like "_%d" where %d is a column number */
@@ -1564,7 +1565,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 		gboolean nullforced = FALSE;
 		isnull = (gchar*)xmlGetProp (xml_field, BAD_CAST "isnull");
 		if (isnull) {
-			if ((*isnull == 't') || (*isnull == 't'))
+			if ((*isnull == 't') || (*isnull == 'T'))
 				nullforced = TRUE;
 			g_free (isnull);
 		}
@@ -1572,13 +1573,15 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 		if (!nullforced) {
 			value = g_new0 (GValue, 1);
 			gchar* nodeval = (gchar*)xmlNodeGetContent (xml_field);
-			if (!gda_value_set_from_string (value, nodeval, gdatype)) {
-				g_free (value);
-  				xmlFree(nodeval);
-				value = gda_value_new_null ();
+			if (nodeval) {
+				if (!gda_value_set_from_string (value, nodeval, gdatype)) {
+					g_free (value);
+					value = gda_value_new_null ();
+				}
+				xmlFree(nodeval);
 			}
-
-			xmlFree(nodeval);
+			else
+				value = gda_value_new_null ();	
 		}
 
 		g_ptr_array_index (values, pos) = value;
@@ -1751,7 +1754,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 	 * node->data is:
 	 * - NULL if the value must be replaced by the value of the copied model
 	 * - a GValue of type GDA_VALYE_TYPE_NULL if a null value must be inserted in the dest data model
-	 * - a GValue of a different type is the value must be converted from the ser data model
+	 * - a GValue of a different type if the value must be converted from the ser data model
 	 */
 	append_types = g_new0 (GType, to_nb_cols);
 	for (plist = copy_params, i = 0; plist; plist = plist->next, i++) {
@@ -1775,6 +1778,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 		else
 			append_values = g_list_prepend (append_values, gda_value_new_null ());
 	}
+	append_values = g_list_reverse (append_values);
 	
 	/* actual data copy (no memory allocation is needed here) */
 	gda_data_model_send_hint (to, GDA_DATA_MODEL_HINT_START_BATCH_UPDATE, NULL);
@@ -1800,7 +1804,8 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 				if (avlist->data) {
 					if (append_types [i] && gda_value_is_null ((GValue *) (avlist->data))) 
 						g_value_init ((GValue *) (avlist->data), append_types [i]);
-					if (!gda_value_is_null (value) && !g_value_transform (value, (GValue *) (avlist->data))) {
+					if (!gda_value_is_null (value) && 
+					    !g_value_transform (value, (GValue *) (avlist->data))) {
 						gchar *str;
 
 						str = gda_value_stringify (value);
@@ -1819,8 +1824,8 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 
 			values = g_list_prepend (values, value);
 
-			plist = g_slist_next (plist);
-			avlist = g_list_next (avlist);
+			plist = plist->next;
+			avlist = avlist->next;
 			i--;
 		}
 
