@@ -118,6 +118,7 @@ gda_holder_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaHolderClass),
 			(GBaseInitFunc) NULL,
@@ -130,7 +131,10 @@ gda_holder_get_type (void)
 			(GInstanceInitFunc) gda_holder_init
 		};
 		
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaHolder", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaHolder", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 
 	return type;
@@ -323,6 +327,8 @@ GdaHolder *
 gda_holder_new_inline (GType type, const gchar *id, ...)
 {
 	GdaHolder *holder;
+
+	static GStaticMutex serial_mutex = G_STATIC_MUTEX_INIT;
 	static guint serial = 0;
 
 	holder = gda_holder_new (type);
@@ -332,8 +338,11 @@ gda_holder_new_inline (GType type, const gchar *id, ...)
 
 		if (id)
 			holder->priv->id = g_strdup (id);
-		else
+		else {
+			g_static_mutex_lock (&serial_mutex);
 			holder->priv->id = g_strdup_printf ("%d", serial++);
+			g_static_mutex_unlock (&serial_mutex);
+		}
 
 		va_start (ap, id);
 		value = gda_value_new (type);
@@ -386,9 +395,6 @@ gda_holder_dispose (GObject *object)
 {
 	GdaHolder *holder;
 
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GDA_IS_HOLDER (object));
-
 	holder = GDA_HOLDER (object);
 	if (holder->priv) {
 		gda_holder_set_bind (holder, NULL);
@@ -765,6 +771,7 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 	gboolean changed = TRUE;
 	gboolean newvalid;
 	const GValue *current_val;
+	gboolean newnull;
 #define DEBUG_HOLDER
 #undef DEBUG_HOLDER
 
@@ -775,27 +782,24 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 	holder->priv->invalid_forced = FALSE;
 
 	/* holder will be changed? */
+	newnull = !value || gda_value_is_null (value);
 	current_val = gda_holder_get_value (holder);
 	if (current_val == value)
 		changed = FALSE;
-	else if ((!current_val || gda_value_is_null ((GValue *)current_val)) && 
-		 (!value       || gda_value_is_null (value))) 
+	else if ((!current_val || gda_value_is_null ((GValue *)current_val)) && newnull)
 		changed = FALSE;
 	else if (value && current_val &&
 		 (G_VALUE_TYPE (value) == G_VALUE_TYPE ((GValue *)current_val)))
-		changed = gda_value_compare (value, (GValue *)current_val);
+		changed = gda_value_bcompare (value, (GValue *)current_val);
 		
 	/* holder's validity */
 	newvalid = TRUE;
-	if (!value || gda_value_is_null (value))
+	if (newnull)
 		if (holder->priv->not_null)
 			newvalid = FALSE;
 
-	if (value &&
-	    (G_VALUE_TYPE (value) != GDA_TYPE_NULL) &&
-	    (G_VALUE_TYPE (value) != holder->priv->g_type)) {
+	if (!newnull && (G_VALUE_TYPE (value) != holder->priv->g_type)) 
 		newvalid = FALSE;
-	}
 
 #ifdef DEBUG_HOLDER
 	g_print ("Changed holder %p (%s): value %s --> %s \t(type %d -> %d) VALID: %d->%d CHANGED: %d\n", 
@@ -819,8 +823,7 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 	/* check is the new value is the default one */
 	holder->priv->default_forced = FALSE;
 	if (holder->priv->default_value) {
-		if ((G_VALUE_TYPE (holder->priv->default_value) == GDA_TYPE_NULL) && 
-		    (!value || gda_value_is_null (value)))
+		if ((G_VALUE_TYPE (holder->priv->default_value) == GDA_TYPE_NULL) && newnull)
 			holder->priv->default_forced = TRUE;
 		else if ((G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) &&
 			 value && (G_VALUE_TYPE (value) == holder->priv->g_type))
@@ -1193,9 +1196,8 @@ gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to)
 		g_signal_connect (G_OBJECT (holder->priv->simple_bind), "changed",
 				  G_CALLBACK (full_bind_changed_cb), holder);
 
-		/* if alias_of has a different value than holder, then we set holder to the new value */
-		if (gda_value_compare_ext (value1, value2))
-			g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
+		/* if bind_to has a different value than holder, then we set holder to the new value */
+		gda_holder_set_value (holder, value2);
 		if (value1)
 			gda_value_free (value1);
 	}
@@ -1226,7 +1228,7 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 
 	/* get a copy of the current values of @holder and @alias_of */
 	if (alias_of) {
-		g_return_if_fail (alias_of && GDA_IS_HOLDER (alias_of));
+		g_return_if_fail (GDA_IS_HOLDER (alias_of));
 		g_return_if_fail (alias_of->priv);
 		g_return_if_fail (holder->priv->g_type == alias_of->priv->g_type);
 		cvalue = gda_holder_get_value (alias_of);
