@@ -25,12 +25,6 @@
 #include <libgda/binreloc/gda-binreloc.h>
 #include <sql-parser/gda-sql-parser.h>
 
-static GStaticRecMutex gda_mutex = G_STATIC_REC_MUTEX_INIT;
-#define GDA_LOCK() g_static_rec_mutex_lock(&gda_mutex)
-#define GDA_UNLOCK() g_static_rec_mutex_unlock(&gda_mutex)
-
-static GMainLoop *main_loop = NULL;
-
 /* global variables */
 xmlDtdPtr       gda_array_dtd = NULL;
 xmlDtdPtr       gda_paramlist_dtd = NULL;
@@ -39,22 +33,24 @@ xmlDtdPtr       gda_server_op_dtd = NULL;
 
 /**
  * gda_init
- * @app_id: name of the program.
- * @version: revision number of the program.
- * @nargs: number of arguments, usually argc from main().
- * @args: list of arguments, usually argv from main().
  * 
- * Initializes the GDA library. 
+ * Initializes the GDA library, must be called prior to any Libgda usage. Note that unless the
+ * LIBGDA_NO_THREADS environment variable is set (to any value), the GLib thread system will
+ * be initialized as well if not yet initialized.
  */
 void
-gda_init (const gchar *app_id, const gchar *version, gint nargs, gchar *args[])
+gda_init (void)
 {
+	static GStaticMutex init_mutex = G_STATIC_MUTEX_INIT;
 	static gboolean initialized = FALSE;
+
 	GType type;
 	gchar *file;
 
+	g_static_mutex_lock (&init_mutex);
 	if (initialized) {
-		gda_log_error (_("Attempt to re-initialize GDA library. ignored."));
+		g_static_mutex_unlock (&init_mutex);
+		gda_log_error (_("Ignoring attempt to re-initialize GDA library."));
 		return;
 	}
 
@@ -74,7 +70,6 @@ gda_init (const gchar *app_id, const gchar *version, gint nargs, gchar *args[])
 	}
 
 	g_type_init ();
-	g_set_prgname (app_id);
 
 	if (!g_module_supported ())
 		g_error (_("libgda needs GModule. Finishing..."));
@@ -107,123 +102,69 @@ gda_init (const gchar *app_id, const gchar *version, gint nargs, gchar *args[])
 	gda_gbr_init ();
 
 	/* array DTD */
+	gda_array_dtd = NULL;
 	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "libgda-array.dtd", NULL);
-	gda_array_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+	if (g_file_test (file, G_FILE_TEST_EXISTS))
+		gda_array_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+	
+	if (!gda_array_dtd) {
+		if (g_getenv ("GDA_TOP_SRC_DIR")) {
+			g_free (file);
+			file = g_build_filename (g_getenv ("GDA_TOP_SRC_DIR"), "libgda", "libgda-array.dtd", NULL);
+			gda_array_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+		}
+		if (!gda_array_dtd)
+			g_message (_("Could not parse '%s': "
+				     "XML data import validation will not be performed (some weird errors may occur)"),
+				   file);
+	}
 	if (gda_array_dtd)
 		gda_array_dtd->name = xmlStrdup((xmlChar*) "gda_array");
-	else
-		g_message (_("Could not parse '%s': "
-			     "XML data import validation will not be performed (some weird errors may occur)"),
-			   file);
 	g_free (file);
 
 	/* paramlist DTD */
+	gda_paramlist_dtd = NULL;
 	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "libgda-paramlist.dtd", NULL);
-	gda_paramlist_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+	if (g_file_test (file, G_FILE_TEST_EXISTS))
+		gda_paramlist_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+
+	if (!gda_paramlist_dtd) {
+		if (g_getenv ("GDA_TOP_SRC_DIR")) {
+			g_free (file);
+			file = g_build_filename (g_getenv ("GDA_TOP_SRC_DIR"), "libgda", "libgda-paramlist.dtd", NULL);
+			gda_paramlist_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+		}
+		if (!gda_paramlist_dtd)
+			g_message (_("Could not parse '%s': "
+				     "XML data import validation will not be performed (some weird errors may occur)"),
+				   file);
+	}
 	if (gda_paramlist_dtd)
 		gda_paramlist_dtd->name = xmlStrdup((xmlChar*) "data-set-spec");
-	else
-		g_message (_("Could not parse '%s': "
-			     "XML data import validation will not be performed (some weird errors may occur)"),
-			   file);
 	g_free (file);
 
 	/* server operation DTD */
+	gda_server_op_dtd = NULL;
 	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "libgda-server-operation.dtd", NULL);
-	gda_server_op_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+	if (g_file_test (file, G_FILE_TEST_EXISTS))
+		gda_server_op_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+
+	if (!gda_server_op_dtd) {
+		if (g_getenv ("GDA_TOP_SRC_DIR")) {
+			g_free (file);
+			file = g_build_filename (g_getenv ("GDA_TOP_SRC_DIR"), "libgda", "libgda-server-operation.dtd", NULL);
+			gda_server_op_dtd = xmlParseDTD (NULL, (xmlChar*)file);
+		}
+		if (!gda_server_op_dtd)
+			g_message (_("Could not parse '%s': "
+				     "Validation for XML files for server operations will not be performed (some weird errors may occur)"),
+				   file);
+	}
+
 	if (gda_server_op_dtd)
 		gda_server_op_dtd->name = xmlStrdup((xmlChar*) "serv_op");
-	else
-		g_message (_("Could not parse '%s': "
-			     "Validation for XML files for server operations will not be performed (some weird errors may occur)"),
-			   file);
 	g_free (file);
 
 	initialized = TRUE;
-}
-
-typedef struct {
-	GdaInitFunc init_func;
-	gpointer user_data;
-} InitCbData;
-
-static gboolean
-idle_cb (gpointer user_data)
-{
-	InitCbData *cb_data = (InitCbData *) user_data;
-
-	g_return_val_if_fail (cb_data != NULL, FALSE);
-
-	if (cb_data->init_func)
-		cb_data->init_func (cb_data->user_data);
-
-	g_free (cb_data);
-
-	return FALSE;
-}
-
-/**
- * gda_main_run
- * @init_func: function to be called when everything has been initialized.
- * @user_data: data to be passed to the init function.
- *
- * Runs the GDA main loop, which is nothing more than the glib main
- * loop, but with internally added stuff specific for applications using
- * libgda.
- *
- * You can specify a function to be called after everything has been correctly
- * initialized (that is, for initializing your own stuff).
- */
-void
-gda_main_run (GdaInitFunc init_func, gpointer user_data)
-{
-	GDA_LOCK ();
-	if (main_loop) {
-		GDA_UNLOCK ();
-		return;
-	}
-
-	if (init_func) {
-		InitCbData *cb_data;
-
-		cb_data = g_new (InitCbData, 1);
-		cb_data->init_func = init_func;
-		cb_data->user_data = user_data;
-
-		g_idle_add ((GSourceFunc) idle_cb, cb_data);
-	}
-
-	main_loop = g_main_loop_new (g_main_context_default (), FALSE);
-	g_main_loop_run (main_loop);
-	GDA_UNLOCK ();
-}
-
-/**
- * gda_main_quit
- * 
- * Exits the main loop.
- */
-void
-gda_main_quit (void)
-{
-	GDA_LOCK ();
-	g_main_loop_quit (main_loop);
-	g_main_loop_unref (main_loop);
-
-	main_loop = NULL;
-	GDA_UNLOCK ();
-}
-
-/*
- * Convenient Functions
- */
-
-
-/* module error */
-GQuark gda_general_error_quark (void)
-{
-	static GQuark quark;
-	if (!quark)
-		quark = g_quark_from_static_string ("gda_general_error");
-	return quark;
+	g_static_mutex_unlock (&init_mutex);
 }
