@@ -38,7 +38,6 @@
 #endif
 #include <glib/gi18n-lib.h>
 
-#define PARENT_TYPE G_TYPE_OBJECT
 #define CLASS(operation) (GDA_SERVER_OPERATION_CLASS (G_OBJECT_GET_CLASS (operation)))
 
 static void gda_server_operation_class_init (GdaServerOperationClass *klass);
@@ -166,17 +165,17 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 	g_object_class_install_property (object_class, PROP_CNC,
 					 g_param_spec_object ("connection", NULL, NULL, 
 							      GDA_TYPE_CONNECTION,
-							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_PROV,
 					 g_param_spec_object ("provider_obj", NULL, NULL, 
-                                                               GDA_TYPE_SERVER_PROVIDER,
-							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+							      GDA_TYPE_SERVER_PROVIDER,
+							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_SPEC_FILE,
 					 g_param_spec_string ("spec_file", NULL, NULL, 
 							      NULL, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_OP_TYPE,
 					 g_param_spec_int ("op_type", NULL, NULL, 
-							   0, GDA_SERVER_OPERATION_NB - 1, 
+							   0, GDA_SERVER_OPERATION_LAST - 1, 
 							   0, G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
 
@@ -243,6 +242,8 @@ gda_server_operation_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+
 		static const GTypeInfo info = {
 			sizeof (GdaServerOperationClass),
 			(GBaseInitFunc) NULL,
@@ -255,7 +256,10 @@ gda_server_operation_get_type (void)
 			(GInstanceInitFunc) gda_server_operation_init
 		};
 
-		type = g_type_register_static (PARENT_TYPE, "GdaServerOperation", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (!type)
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaServerOperation", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -412,6 +416,7 @@ node_find_or_create (GdaServerOperation *op, const gchar *path)
 				node = NULL;
 				break;
 		}
+		g_free(cpath);
 	}
 
 	/*g_print ("# %s (%s) => %p\n", __FUNCTION__, path, node);*/
@@ -511,7 +516,7 @@ gda_server_operation_set_property (GObject *object,
 			if (op->priv->cnc)
 				g_object_unref (op->priv->cnc);
 
-			op->priv->cnc = GDA_CONNECTION( g_value_get_object (value));
+			op->priv->cnc = GDA_CONNECTION (g_value_get_object (value));
 			op->priv->cnc_set = TRUE;
 
 			if (op->priv->cnc) {
@@ -641,6 +646,12 @@ gda_server_operation_get_property (GObject *object,
 	op = GDA_SERVER_OPERATION (object);
 	if (op->priv) {
 		switch (param_id) {
+		case PROP_CNC:
+			g_value_set_object (value, op->priv->cnc);
+			break;
+		case PROP_PROV:
+			g_value_set_object (value, op->priv->prov);
+			break;
 		case PROP_OP_TYPE:
 			g_value_set_int (value, op->priv->op_type);
 			break;
@@ -1116,7 +1127,8 @@ gda_server_operation_op_type_to_string (GdaServerOperationType type)
 	case GDA_SERVER_OPERATION_DROP_VIEW:
 		return "DROP_VIEW";
 	default:
-		g_error (_("Non handled GdaServerOperationType, please report error"));
+		g_error (_("Non handled GdaServerOperationType, please report error to "
+			   "http://bugzilla.gnome.org/ for the \"libgda\" product"));
 		return "";
 	}
 }
@@ -1288,7 +1300,7 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 	/* actual data loading */
 	if (strcmp ((gchar*)node->name, "serv_op_data")) {
 		g_set_error (error, 0, 0,
-			     _("Expected tag <serv_op_data>, got <%s>"), node->name);
+			     _("Expected tag <%s>, got <%s>"), "serv_op_data", node->name);
 		return FALSE;
 	}
 	
@@ -1302,7 +1314,7 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 
 		if (strcmp ((gchar*)cur->name, "op_data")) {
 			g_set_error (error, 0, 0,
-				     _("Expected tag <op_data>, got <%s>"), cur->name);
+				     _("Expected tag <%s>, got <%s>"), "op_data", cur->name);
 			return FALSE;
 		}
 
