@@ -114,6 +114,7 @@ gda_server_provider_class_init (GdaServerProviderClass *klass)
 	klass->cancel = NULL;
 	klass->create_connection = NULL;
 	memset (&(klass->meta_funcs), 0, sizeof (GdaServerProviderMeta));
+	klass->xa_funcs = NULL;
 
 	 /* Properties */
         object_class->set_property = gda_server_provider_set_property;
@@ -191,6 +192,7 @@ gda_server_provider_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaServerProviderClass),
 			(GBaseInitFunc) NULL,
@@ -202,7 +204,10 @@ gda_server_provider_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_server_provider_init
 		};
-		type = g_type_register_static (PARENT_TYPE, "GdaServerProvider", &info, G_TYPE_FLAG_ABSTRACT);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (PARENT_TYPE, "GdaServerProvider", &info, G_TYPE_FLAG_ABSTRACT);
+		g_static_mutex_unlock (&registering);
 	}
 
 	return type;
@@ -404,8 +409,8 @@ static OpReq op_req_DROP_VIEW [] = {
  * Creates a new #GdaServerOperation object which can be modified in order to perform the @type type of
  * action. The @options can contain:
  * <itemizedlist>
- *  <listitem>parameters which ID is a path in the resulting GdaServerOperation object, to initialize some value</listitem>
- *  <listitem>parameters which may change the contents of the GdaServerOperation, see <link linkend="gda-server-op-information-std">this section</link> for more information</listitem>
+ *  <listitem>named values which ID is a path in the resulting GdaServerOperation object, to initialize some value</listitem>
+ *  <listitem>named values which may change the contents of the GdaServerOperation, see <link linkend="gda-server-op-information-std">this section</link> for more information</listitem>
  * </itemizedlist>
  *
  * Returns: a new #GdaServerOperation object, or %NULL in the provider does not support the @type type
@@ -416,10 +421,12 @@ gda_server_provider_create_operation (GdaServerProvider *provider, GdaConnection
 				      GdaServerOperationType type, 
 				      GdaSet *options, GError **error)
 {
+	static GStaticMutex init_mutex = G_STATIC_MUTEX_INIT;
 	static OpReq **op_req_table = NULL;
 
+	g_static_mutex_lock (&init_mutex);
 	if (! op_req_table) {
-		op_req_table = g_new0 (OpReq *, GDA_SERVER_OPERATION_NB);
+		op_req_table = g_new0 (OpReq *, GDA_SERVER_OPERATION_LAST);
 
 		op_req_table [GDA_SERVER_OPERATION_CREATE_DB] = op_req_CREATE_DB;
 		op_req_table [GDA_SERVER_OPERATION_DROP_DB] = op_req_DROP_DB;
@@ -437,6 +444,7 @@ gda_server_provider_create_operation (GdaServerProvider *provider, GdaConnection
 		op_req_table [GDA_SERVER_OPERATION_CREATE_VIEW] = op_req_CREATE_VIEW;
 		op_req_table [GDA_SERVER_OPERATION_DROP_VIEW] = op_req_DROP_VIEW;
 	}
+	g_static_mutex_unlock (&init_mutex);
 
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
 
@@ -524,7 +532,8 @@ gda_server_provider_render_operation (GdaServerProvider *provider, GdaConnection
  * @op: a #GdaServerOperation object
  * @error: a place to store an error, or %NULL
  *
- * Performs the operation described by @op.
+ * Performs the operation described by @op. Note that @op is not destroyed by this method
+ * and can be reused.
  *
  * Returns: TRUE if no error occurred
  */
@@ -820,8 +829,10 @@ gda_server_provider_escape_string (GdaServerProvider *provider, GdaConnection *c
 
 	if (CLASS (provider)->escape_string) {
 		if (! CLASS (provider)->unescape_string)
-			g_warning (_("GdaServerProvider object implements the escape_string() virtual method but "
-				     "does not implement the unescape_string() one, please report this bug."));
+			g_warning (_("GdaServerProvider object implements the %s virtual method but "
+				     "does not implement the %s one, please report this bug to "
+				     "http://bugzilla.gnome.org/ for the \"libgda\" product."), 
+				   "escape_string()", "unescape_string()");
 		return (CLASS (provider)->escape_string)(provider, cnc, str);
 	}
 	else 
@@ -847,8 +858,10 @@ gda_server_provider_unescape_string (GdaServerProvider *provider, GdaConnection
 
 	if (CLASS (provider)->unescape_string) {
 		if (! CLASS (provider)->escape_string)
-			g_warning (_("GdaServerProvider object implements the unescape_string() virtual method but "
-				     "does not implement the escape_string() one, please report this bug."));
+			g_warning (_("GdaServerProvider object implements the %s virtual method but "
+				     "does not implement the %s one, please report this bug to "
+				     "http://bugzilla.gnome.org/ for the \"libgda\" product."),
+				   "unescape_string()", "escape_string()");
 		return (CLASS (provider)->unescape_string)(provider, cnc, str);
 	}
 	else
