@@ -97,6 +97,7 @@ gda_statement_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaStatementClass),
 			(GBaseInitFunc) NULL,
@@ -109,7 +110,10 @@ gda_statement_get_type (void)
 			(GInstanceInitFunc) gda_statement_init
 		};
 		
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaStatement", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaStatement", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -364,7 +368,7 @@ gda_statement_check_structure (GdaStatement *stmt, GError **error)
 }
 
 /**
- * gda_statement_check_connection
+ * gda_statement_check_validity
  * @stmt: a #GdaStatement object
  * @cnc: a #GdaConnection object, or %NULL
  * @error: a place to store errors, or %NULL
@@ -374,22 +378,45 @@ gda_statement_check_structure (GdaStatement *stmt, GError **error)
  *
  * If @cnc is %NULL, then cleans anything related to @cnc in @stmt.
  *
+ * See gda_sql_statement_check_validity() for more information.
+ *
  * Returns: TRUE if every object actually exists in @cnc's database
  */
 gboolean
-gda_statement_check_connection (GdaStatement *stmt, GdaConnection *cnc, GError **error)
+gda_statement_check_validity (GdaStatement *stmt, GdaConnection *cnc, GError **error)
 {
 	gboolean retval;
 	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), FALSE);
 	g_return_val_if_fail (stmt->priv, FALSE);
 	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), FALSE);
 
-	retval = gda_sql_statement_check_connection (stmt->priv->internal_struct, cnc, error);
+	retval = gda_sql_statement_check_validity (stmt->priv->internal_struct, cnc, error);
 	g_signal_emit (stmt, gda_statement_signals [CHECKED], 0, cnc, retval);
 
 	return retval;
 }
 
+/**
+ * gda_statement_normalize
+ * @stmt: a #GdaStatement object
+ * @cnc: a #GdaConnection object
+ * @error: a place to store errors, or %NULL
+ *
+ * "Normalizes" some parts of @stmt, see gda_sql_statement_normalize() for more
+ * information.
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_statement_normalize (GdaStatement *stmt, GdaConnection *cnc, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), FALSE);
+	g_return_val_if_fail (stmt->priv, FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+
+	return gda_sql_statement_normalize (stmt->priv->internal_struct, cnc, error);
+}
+
 /**
  * gda_statement_serialize
  * @stmt: a #GdaStatement object
@@ -1106,7 +1133,8 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 			      GDA_STATEMENT_SQL_PARAMS_SHORT |
 			      GDA_STATEMENT_SQL_PARAMS_AS_COLON |
 			      GDA_STATEMENT_SQL_PARAMS_AS_DOLLAR |
-			      GDA_STATEMENT_SQL_PARAMS_AS_QMARK))
+			      GDA_STATEMENT_SQL_PARAMS_AS_QMARK |
+			      GDA_STATEMENT_SQL_PARAMS_AS_UQMARK))
 		render_pspec = TRUE;
 
 	if (is_default)
@@ -1128,7 +1156,9 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 	}
 	if (!h && 
 	    (!render_pspec ||
-	     (context->flags & (GDA_STATEMENT_SQL_PARAMS_AS_DOLLAR | GDA_STATEMENT_SQL_PARAMS_AS_QMARK)))) {
+	     (context->flags & (GDA_STATEMENT_SQL_PARAMS_AS_DOLLAR | 
+				GDA_STATEMENT_SQL_PARAMS_AS_QMARK |
+				GDA_STATEMENT_SQL_PARAMS_AS_UQMARK)))) {
 		/* a real value is needed or @context->params_used needs to be correct, and no GdaHolder found */
 		g_set_error (error, GDA_STATEMENT_ERROR, GDA_STATEMENT_PARAM_ERROR,
 			     _("Missing parameter '%s'"), pspec->name);
@@ -1173,11 +1203,15 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 		g_string_append_printf (string, ":%s", str);
 		g_free (str);
 	}
-	else if (context->flags & (GDA_STATEMENT_SQL_PARAMS_AS_DOLLAR | GDA_STATEMENT_SQL_PARAMS_AS_QMARK)) {
+	else if (context->flags & (GDA_STATEMENT_SQL_PARAMS_AS_DOLLAR | 
+				   GDA_STATEMENT_SQL_PARAMS_AS_QMARK |
+				   GDA_STATEMENT_SQL_PARAMS_AS_UQMARK)) {
 		if (context->flags & GDA_STATEMENT_SQL_PARAMS_AS_DOLLAR)
 			g_string_append_printf (string, "$%d", g_slist_length (context->params_used));
-		else
+		else if (context->flags & GDA_STATEMENT_SQL_PARAMS_AS_QMARK)
 			g_string_append_printf (string, "?%d", g_slist_length (context->params_used));
+		else
+			g_string_append_c (string, '?');
 	}
 	else {
 		GdaStatementSqlFlag flag = context->flags;
@@ -1431,86 +1465,86 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 
 	str = NULL;
 	switch (op->operator) {
-	case GDA_SQL_OPERATOR_EQ:
+	case GDA_SQL_OPERATOR_TYPE_EQ:
 		if (SQL_OPERAND (sql_list->next->data)->is_null) 
 			str = g_strdup_printf ("%s IS NULL", SQL_OPERAND (sql_list->data)->sql);
 		else
 			str = g_strdup_printf ("%s = %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_IS:
+	case GDA_SQL_OPERATOR_TYPE_IS:
 		str = g_strdup_printf ("%s IS %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_LIKE:
+	case GDA_SQL_OPERATOR_TYPE_LIKE:
 		str = g_strdup_printf ("%s LIKE %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_GT:
+	case GDA_SQL_OPERATOR_TYPE_GT:
 		str = g_strdup_printf ("%s > %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_LT:
+	case GDA_SQL_OPERATOR_TYPE_LT:
 		str = g_strdup_printf ("%s < %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_GEQ:
+	case GDA_SQL_OPERATOR_TYPE_GEQ:
 		str = g_strdup_printf ("%s >= %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_LEQ:
+	case GDA_SQL_OPERATOR_TYPE_LEQ:
 		str = g_strdup_printf ("%s <= %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_DIFF:
+	case GDA_SQL_OPERATOR_TYPE_DIFF:
 		str = g_strdup_printf ("%s != %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_REGEXP:
+	case GDA_SQL_OPERATOR_TYPE_REGEXP:
 		str = g_strdup_printf ("%s ~ %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_REGEXP_CI:
+	case GDA_SQL_OPERATOR_TYPE_REGEXP_CI:
 		str = g_strdup_printf ("%s ~* %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_NOT_REGEXP:
+	case GDA_SQL_OPERATOR_TYPE_NOT_REGEXP:
 		str = g_strdup_printf ("%s !~ %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_NOT_REGEXP_CI:
+	case GDA_SQL_OPERATOR_TYPE_NOT_REGEXP_CI:
 		str = g_strdup_printf ("%s !~* %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_SIMILAR:
+	case GDA_SQL_OPERATOR_TYPE_SIMILAR:
 		str = g_strdup_printf ("%s SIMILAR TO %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_REM:
+	case GDA_SQL_OPERATOR_TYPE_REM:
 		str = g_strdup_printf ("%s %% %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_DIV:
+	case GDA_SQL_OPERATOR_TYPE_DIV:
 		str = g_strdup_printf ("%s / %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_BITAND:
+	case GDA_SQL_OPERATOR_TYPE_BITAND:
 		str = g_strdup_printf ("%s & %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_BITOR:
+	case GDA_SQL_OPERATOR_TYPE_BITOR:
 		str = g_strdup_printf ("%s | %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_BETWEEN:
+	case GDA_SQL_OPERATOR_TYPE_BETWEEN:
 		str = g_strdup_printf ("%s BETWEEN %s AND %s", SQL_OPERAND (sql_list->data)->sql, 
 				       SQL_OPERAND (sql_list->next->data)->sql,
 				       SQL_OPERAND (sql_list->next->next->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_ISNULL:
+	case GDA_SQL_OPERATOR_TYPE_ISNULL:
 		str = g_strdup_printf ("%s IS NULL", SQL_OPERAND (sql_list->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_ISNOTNULL:
+	case GDA_SQL_OPERATOR_TYPE_ISNOTNULL:
 		str = g_strdup_printf ("%s IS NOT NULL", SQL_OPERAND (sql_list->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_BITNOT:
+	case GDA_SQL_OPERATOR_TYPE_BITNOT:
 		str = g_strdup_printf ("~ %s", SQL_OPERAND (sql_list->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_NOT:
+	case GDA_SQL_OPERATOR_TYPE_NOT:
 		str = g_strdup_printf ("NOT %s", SQL_OPERAND (sql_list->data)->sql);
 		break;
-	case GDA_SQL_OPERATOR_IN:
-	case GDA_SQL_OPERATOR_NOTIN: {
+	case GDA_SQL_OPERATOR_TYPE_IN:
+	case GDA_SQL_OPERATOR_TYPE_NOTIN: {
 		gboolean add_p = TRUE;
 		if (sql_list->next && !(sql_list->next->next) &&
 		    *(SQL_OPERAND (sql_list->next->data)->sql)=='(')
 			add_p = FALSE;
 
 		string = g_string_new (SQL_OPERAND (sql_list->data)->sql);
-		if (op->operator == GDA_SQL_OPERATOR_IN)
+		if (op->operator == GDA_SQL_OPERATOR_TYPE_IN)
 			g_string_append (string, " IN ");
 		else
 			g_string_append (string, " NOT IN ");
@@ -1527,22 +1561,22 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 		g_string_free (string, FALSE);
 		break;
 	}
-	case GDA_SQL_OPERATOR_CONCAT:
+	case GDA_SQL_OPERATOR_TYPE_CONCAT:
 		multi_op = "||";
 		break;
-	case GDA_SQL_OPERATOR_PLUS:
+	case GDA_SQL_OPERATOR_TYPE_PLUS:
 		multi_op = "+";
 		break;
-	case GDA_SQL_OPERATOR_MINUS:
+	case GDA_SQL_OPERATOR_TYPE_MINUS:
 		multi_op = "-";
 		break;
-	case GDA_SQL_OPERATOR_STAR:
+	case GDA_SQL_OPERATOR_TYPE_STAR:
 		multi_op = "*";
 		break;
-	case GDA_SQL_OPERATOR_AND:
+	case GDA_SQL_OPERATOR_TYPE_AND:
 		multi_op = "AND";
 		break;
-	case GDA_SQL_OPERATOR_OR:
+	case GDA_SQL_OPERATOR_TYPE_OR:
 		multi_op = "OR";
 		break;
 	default:
