@@ -38,39 +38,46 @@ G_BEGIN_DECLS
 const gchar *gda_g_type_to_string (GType type);
 GType        gda_g_type_from_string (const gchar *str);
 
-GList       *gda_string_hash_to_list (GHashTable *hash_table);
-
 /* 
  * SQL escaping
  */
 gchar       *gda_default_escape_string (const gchar *string);
 gchar       *gda_default_unescape_string (const gchar *string);
-
-/*
- * File management utility functions
- */
-gchar    *gda_file_load (const gchar *filename);
-gboolean  gda_file_save (const gchar *filename, const gchar *buffer, gint len);
-
-/*
- * XML Id encoding and decoding
- */
-gchar *gda_utility_build_encoded_id (const gchar *prefix, const gchar *id);
-gchar *gda_utility_build_decoded_id (const gchar *prefix, const gchar *id);
+guint        gda_identifier_hash (const gchar *id);
+gboolean     gda_identifier_equal (const gchar *id1, const gchar *id2);
+gchar      **gda_completion_list_get (GdaConnection *cnc, const gchar *text, gint start, gint end);
 
 /*
  * Param & model utilities
  */
-gboolean gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...);
-void     gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
+gboolean     gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...);
+void         gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
 					      const gint *cols, gint nb_cols, const gint *rows, gint nb_rows,
 					      gboolean use_col_ids);
-void     gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *sources);
+void         gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *sources);
 
-/* translate any text to an alphanumerical text */
-gchar *gda_text_to_alphanum (const gchar *text);
-gchar *gda_alphanum_to_text (gchar *text);
+/* 
+ * translate any text to an alphanumerical text 
+ */
+gchar       *gda_text_to_alphanum (const gchar *text);
+gchar       *gda_alphanum_to_text (gchar *text);
 
+/*
+ * Statement computation from meta store 
+ */
+gboolean     gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gboolean require_pk, 
+					 GdaStatement **insert_stmt, GdaStatement **update_stmt, GdaStatement **delete_stmt, 
+					 GError **error);
+
+/*
+ * DSN and connection string manipulations
+ */
+gchar       *gda_rfc1738_encode          (const gchar *string);
+gboolean     gda_rfc1738_decode          (gchar *string);
+void         gda_dsn_split               (const gchar *string, gchar **out_dsn, 
+					  gchar **out_username, gchar **out_password);
+void         gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar **out_provider, 
+					  gchar **out_username, gchar **out_password);
 G_END_DECLS
 
 #endif
