@@ -1,6 +1,6 @@
 /* gda-handler-numerical.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba
+ * Copyright (C) 2003 - 2008 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -58,6 +58,7 @@ gda_handler_numerical_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerNumericalClass),
 			(GBaseInitFunc) NULL,
@@ -76,8 +77,12 @@ gda_handler_numerical_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (GDA_TYPE_OBJECT, "GdaHandlerNumerical", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerNumerical", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -127,8 +132,8 @@ gda_handler_numerical_init (GdaHandlerNumerical * hdl)
         hdl->priv->valid_g_types[9] = G_TYPE_UCHAR;
 	hdl->priv->valid_g_types[10] = G_TYPE_UINT;
 
-	gda_object_set_name (GDA_OBJECT (hdl), _("InternalNumerical"));
-	gda_object_set_description (GDA_OBJECT (hdl), _("Numerical representation"));
+	g_object_set_data (G_OBJECT (hdl), "name", "InternalNumerical");
+	g_object_set_data (G_OBJECT (hdl), "descr", _("Numerical representation"));
 }
 
 static void
@@ -142,8 +147,6 @@ gda_handler_numerical_dispose (GObject *object)
 	hdl = GDA_HANDLER_NUMERICAL (object);
 
 	if (hdl->priv) {
-		gda_object_destroy_check (GDA_OBJECT (object));
-
 		g_free (hdl->priv->valid_g_types);
 		hdl->priv->valid_g_types = NULL;
 
@@ -167,7 +170,7 @@ gda_handler_numerical_new (void)
 {
 	GObject *obj;
 
-	obj = g_object_new (GDA_TYPE_HANDLER_NUMERICAL, "dict", NULL, NULL);
+	obj = g_object_new (GDA_TYPE_HANDLER_NUMERICAL, NULL);
 
 	return (GdaDataHandler *) obj;
 }
@@ -379,5 +382,5 @@ gda_handler_numerical_get_descr (GdaDataHandler *iface)
 	hdl = GDA_HANDLER_NUMERICAL (iface);
 	g_return_val_if_fail (hdl->priv, NULL);
 
-	return gda_object_get_description (GDA_OBJECT (hdl));
+	return g_object_get_data (G_OBJECT (hdl), "descr");
 }
