@@ -57,6 +57,7 @@ gda_handler_type_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerTypeClass),
 			(GBaseInitFunc) NULL,
@@ -75,8 +76,12 @@ gda_handler_type_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerType", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerType", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -116,7 +121,7 @@ gda_handler_type_init (GdaHandlerType * hdl)
 	hdl->priv->valid_g_types = g_new0 (GType, 1);
 	hdl->priv->valid_g_types[0] = G_TYPE_ULONG;
 
-	g_object_set_data (G_OBJECT (hdl), "descr", _("InternalType"));
+	g_object_set_data (G_OBJECT (hdl), "descr", "InternalType");
 	g_object_set_data (G_OBJECT (hdl), "descr", _("Gda type representation"));
 }
 
