@@ -47,6 +47,8 @@ static void gda_sql_parser_get_property (GObject *object,
 					GParamSpec *pspec);
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
+static GStaticRecMutex usage_mutex = G_STATIC_REC_MUTEX_INIT;
+
 
 static void gda_sql_parser_reset (GdaSqlParser *parser);
 static GValue *tokenizer_get_next_token (GdaSqlParser *parser); 
@@ -106,6 +108,7 @@ gda_sql_parser_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaSqlParserClass),
 			(GBaseInitFunc) NULL,
@@ -118,7 +121,10 @@ gda_sql_parser_get_type (void)
 			(GInstanceInitFunc) gda_sql_parser_init
 		};
 		
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaSqlParser", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaSqlParser", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -277,6 +283,7 @@ gda_sql_parser_finalize (GObject *object)
 		
 		klass = (GdaSqlParserClass*) G_OBJECT_GET_CLASS (parser);
 		gda_sql_parser_reset (parser);
+		g_free (parser->priv->context);
 
 		if (klass->delim_alloc) {
 			g_assert (klass->delim_free);
@@ -413,6 +420,8 @@ gda_sql_parser_parse_string (GdaSqlParser *parser, const gchar *sql, const gchar
 	if (!sql)
 		return NULL;
 
+	g_static_rec_mutex_lock (&usage_mutex);
+
 	if (remain)
 		*remain = NULL;
 	
@@ -596,6 +605,8 @@ gda_sql_parser_parse_string (GdaSqlParser *parser, const gchar *sql, const gchar
 
 	parser->priv->mode = parse_mode;
 
+	g_static_rec_mutex_unlock (&usage_mutex);
+
 	return stmt;
 }
 
@@ -607,7 +618,7 @@ gda_sql_parser_parse_string (GdaSqlParser *parser, const gchar *sql, const gchar
  * @error: location to store error, or %NULL
  *
  * Parse @sql and creates a #GdaBatch object which contains all the #GdaStatement objects created while parsing (one object
- * per SQL statement).
+ * per SQL statement). Empty statements (composed of spaces only) do not appear in the resulting object.
  *
  * @sql is parsed and #GdaStatement objects are created as long as no error is found in @sql. If an error is found 
  * at some point, then the parsing stops and @remain may contain a non %NULL pointer, @error may be set, and %NULL
@@ -638,6 +649,8 @@ gda_sql_parser_parse_string_as_batch (GdaSqlParser *parser, const gchar *sql, co
 	if (!sql)
 		return batch;
 
+	g_static_rec_mutex_lock (&usage_mutex);
+
 	int_sql = sql;
 	while (int_sql && allok) {
 		GError *lerror = NULL;
@@ -679,6 +692,8 @@ gda_sql_parser_parse_string_as_batch (GdaSqlParser *parser, const gchar *sql, co
 		g_object_unref (batch);
 		batch = NULL;
 	}
+
+	g_static_rec_mutex_unlock (&usage_mutex);
 	
 	return batch;
 }
@@ -842,10 +857,12 @@ str_casehash (gconstpointer v)
 static gint
 keywordCode (GdaSqlParser *parser, gchar *str, gint len)
 {
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 	static GHashTable *keywords = NULL;
 	gint type;
 	gchar oldc;
 
+	g_static_mutex_lock (&mutex);
 	if (!keywords) {
 		/* if keyword begins with a number, it is the GdaSqlParserFlavour to which it only applies:
 		* for example "4start" would refer to the START keyword for PostgreSQL only */
@@ -929,7 +946,8 @@ keywordCode (GdaSqlParser *parser, gchar *str, gint len)
 		g_hash_table_insert (keywords, "work", GINT_TO_POINTER (L_TRANSACTION));
 		g_hash_table_insert (keywords, "write", GINT_TO_POINTER (L_WRITE));
 	}
-	
+	g_static_mutex_unlock (&mutex);
+
 	oldc = str[len];
 	str[len] = 0;
 	type = GPOINTER_TO_INT (g_hash_table_lookup (keywords, str));
@@ -988,8 +1006,8 @@ getToken (GdaSqlParser *parser)
 	GValue *retval = NULL;
 	gint consumed_chars = 1;
 
-	/* safe init */
-	parser->priv->context->token_type = L_ILLEGAL;
+	/* init to capture non treaded cases */
+	parser->priv->context->token_type = G_MININT;
 
 	if (!*z) {
 		parser->priv->context->token_type = L_END_OF_FILE;
@@ -1049,20 +1067,19 @@ getToken (GdaSqlParser *parser)
 		if (parser->priv->mode != GDA_SQL_PARSER_MODE_DELIMIT) {
 			parser->priv->context->token_type = L_PLUS;
 			consumed_chars = 1;
-			break;
 		}
+		break;
 	case '*': 
 		if (z[1] == '/') {
 			parser->priv->context->token_type = L_RSBRACKET;
 			consumed_chars = 2;
 			parser->priv->context->in_param_spec = FALSE;
-			break;
 		}
 		else {
 			parser->priv->context->token_type = L_STAR;
 			consumed_chars = 1;
-			break;
 		}
+		break;
 	case '%':
 		if (parser->priv->mode != GDA_SQL_PARSER_MODE_DELIMIT) {
 			parser->priv->context->token_type = L_REM;
@@ -1074,7 +1091,6 @@ getToken (GdaSqlParser *parser)
 			if (z[1] != '*' || z[2] == 0) {
 				parser->priv->context->token_type = L_SLASH;
 				consumed_chars = 1;
-				break;
 			}
 		}
 		else if (z[1] == '*') {
@@ -1082,25 +1098,28 @@ getToken (GdaSqlParser *parser)
 			parser->priv->context->token_type = L_LSBRACKET;
 			consumed_chars = 2;
 			parser->priv->context->in_param_spec = TRUE;
-			break;
 		}
+		break;
 	case '=': 
 		if (parser->priv->mode != GDA_SQL_PARSER_MODE_DELIMIT) {
 			parser->priv->context->token_type = L_EQ;
 			consumed_chars = 1 + (z[1] == '=');
-			break;
 		}
+		break;
 	case '<': 
 		if ((c = z[1]) == '=') {
 			parser->priv->context->token_type = L_LEQ;
 			consumed_chars = 2;
-		} else if (c == '>') {
+		}
+		else if (c == '>') {
 			parser->priv->context->token_type = L_DIFF;
 			consumed_chars = 2;
-		} else if (c== '<') {
+		}
+		else if (c== '<') {
 			parser->priv->context->token_type = L_LSHIFT;
 			consumed_chars = 2;
-		} else {
+		}
+		else {
 			parser->priv->context->token_type = L_LT;
 			consumed_chars = 1;
 		}
@@ -1109,10 +1128,12 @@ getToken (GdaSqlParser *parser)
 		if ((c = z[1]) == '=') {
 			parser->priv->context->token_type = L_GEQ;
 			consumed_chars = 2;
-		} else if (c == '>') {
+		}
+		else if (c == '>') {
 			parser->priv->context->token_type = L_RSHIFT;
 			consumed_chars = 2;
-		} else {
+		}
+		else {
 			parser->priv->context->token_type = L_GT;
 			consumed_chars = 1;
 		}
@@ -1133,8 +1154,8 @@ getToken (GdaSqlParser *parser)
 					consumed_chars = 2;
 				}
 			}
-			break;
 		}
+		break;
 	case '|': 
 		if (parser->priv->mode != GDA_SQL_PARSER_MODE_DELIMIT) {
 			if (z[1] != '|') {
@@ -1144,13 +1165,12 @@ getToken (GdaSqlParser *parser)
 				parser->priv->context->token_type = L_CONCAT;
 				consumed_chars = 2;
 			}
-			break;
 		}
 		else {
 			parser->priv->context->token_type = L_RAWSTRING;
 			consumed_chars = 1;
-			break;
 		}
+		break;
 	case ',': 
 		parser->priv->context->token_type = L_COMMA;
 		consumed_chars = 1;
@@ -1159,8 +1179,8 @@ getToken (GdaSqlParser *parser)
 		if (parser->priv->mode != GDA_SQL_PARSER_MODE_DELIMIT) {
 			parser->priv->context->token_type = L_BITAND;
 			consumed_chars = 1;
-			break;
 		}
+		break;
 	case '~': 
 		if (parser->priv->mode != GDA_SQL_PARSER_MODE_DELIMIT) {
 			if (z[1] == '*') {
@@ -1174,8 +1194,8 @@ getToken (GdaSqlParser *parser)
 					parser->priv->context->token_type = L_BITNOT;
 				consumed_chars = 1;
 			}
-			break;
 		}
+		break;
 	case '\'':
 	case '"': {
 		char delim = z[0];
@@ -1242,8 +1262,8 @@ getToken (GdaSqlParser *parser)
 			parser->priv->context->token_type = L_SIMPLEPARAM;
 			retval = token_as_string (parser->priv->context->next_token_start + 1, i - 1);
 			consumed_chars = i;
-			break;
 		}
+		break;
 	case '#': {
 		if (z[1] == '#') {
 			for (i=2; (z[i]) && (IdChar (z[i]) || (z[i] == '+') || (z[i] == '-') || (z[i] == '.') || (z[i] == ':')) &&
@@ -1290,7 +1310,6 @@ getToken (GdaSqlParser *parser)
 					}
 					i++;
 				}
-				break; /* case */
 			}
 		}
 		else if (parser->priv->flavour == GDA_SQL_PARSER_FLAVOUR_SQLITE) {
@@ -1298,21 +1317,19 @@ getToken (GdaSqlParser *parser)
 			parser->priv->context->token_type = L_SIMPLEPARAM;
 			retval = token_as_string (parser->priv->context->next_token_start + 1, i - 1);
 			consumed_chars = i;
-			break;
 		}
+		break;
 	case '@':
 	case ':':
-		if (parser->priv->flavour == GDA_SQL_PARSER_FLAVOUR_SQLITE) {
+		if (z[1] == ':') {
+			parser->priv->context->token_type = L_PGCAST;
+			consumed_chars = 2;
+		}
+		else if (parser->priv->flavour == GDA_SQL_PARSER_FLAVOUR_SQLITE) {
 			for(i=1; isalnum(z[i]) || (z[i] == '_'); i++){}
 			parser->priv->context->token_type = L_SIMPLEPARAM;
 			retval = token_as_string (parser->priv->context->next_token_start + 1, i - 1);
 			consumed_chars = i;
-			break;
-		}
-		else if ((parser->priv->flavour == GDA_SQL_PARSER_FLAVOUR_POSTGRESQL) && (z[1] == ':')) {
-			parser->priv->context->token_type = L_PGCAST;
-			consumed_chars = 2;
-			break;
 		}
 		break;
 #ifndef SQLITE_OMIT_BLOB_LITERAL
@@ -1333,18 +1350,22 @@ getToken (GdaSqlParser *parser)
 			}
 			if ( c ) i++;
 			consumed_chars = i;
-			break;
 		}
-		/* Otherwise fall through to the next case */
+		break;
 	}
 #endif
-	default: 
+	default:
+		break;
+	}
+
+	if (parser->priv->context->token_type == G_MININT) {
+		/* now treat non treated cases */
 		if ((parser->priv->mode != GDA_SQL_PARSER_MODE_DELIMIT) && (! parser->priv->context->in_param_spec)) {
-			if (!IdChar (*z))
-				break;
-			for (i=1; IdChar (z[i]); i++){}
-			parser->priv->context->token_type = keywordCode (parser, (char*)z, i);
-			consumed_chars = i;
+			if (IdChar (*z)) {
+				for (i=1; IdChar (z[i]); i++){}
+				parser->priv->context->token_type = keywordCode (parser, (char*)z, i);
+				consumed_chars = i;
+			}
 		}
 		else {
 			if ((! parser->priv->context->in_param_spec) && IdChar (*z)) {
@@ -1355,47 +1376,51 @@ getToken (GdaSqlParser *parser)
 				if (ttype != L_RAWSTRING) {
 					parser->priv->context->token_type = ttype;
 					consumed_chars = i;
-					break;
 				}
 			}
 
-			if (!strncmp (parser->priv->context->next_token_start, "name:", 5)) {
-				parser->priv->context->next_token_start += 5;
-				retval = getToken (parser);
-				parser->priv->context->token_type = L_PNAME;
-				consumed_chars = 0;
-			}
-			else if (!strncmp (parser->priv->context->next_token_start, "type:", 5)) {
-				parser->priv->context->next_token_start += 5;
-				retval = getToken (parser);
-				parser->priv->context->token_type = L_PTYPE;
-				consumed_chars = 0;
-			}
-			else if (!strncmp (parser->priv->context->next_token_start, "descr:", 6)) {
-				parser->priv->context->next_token_start += 6;
-				retval = getToken (parser);
-				parser->priv->context->token_type = L_PDESCR;
-				consumed_chars = 0;
-			}
-			else if (!strncmp (parser->priv->context->next_token_start, "nullok:", 7)) {
-				parser->priv->context->next_token_start += 7;
-				retval = getToken (parser);
-				parser->priv->context->token_type = L_PNULLOK;
-				consumed_chars = 0;
-			}
-			else {
-				for (i=1; z[i] && (! isspace (z[i])) && 
-					     (z[i] != parser->priv->context->delimiter) && (z[i] != '*') && 
-					     (z[i] != '\'') && (z[i] != '"'); i++){}
-				parser->priv->context->token_type = L_RAWSTRING;
-				consumed_chars = i;
+			if (parser->priv->context->token_type == G_MININT) {
+				if (!strncmp (parser->priv->context->next_token_start, "name:", 5)) {
+					parser->priv->context->next_token_start += 5;
+					retval = getToken (parser);
+					parser->priv->context->token_type = L_PNAME;
+					consumed_chars = 0;
+				}
+				else if (!strncmp (parser->priv->context->next_token_start, "type:", 5)) {
+					parser->priv->context->next_token_start += 5;
+					retval = getToken (parser);
+					parser->priv->context->token_type = L_PTYPE;
+					consumed_chars = 0;
+				}
+				else if (!strncmp (parser->priv->context->next_token_start, "descr:", 6)) {
+					parser->priv->context->next_token_start += 6;
+					retval = getToken (parser);
+					parser->priv->context->token_type = L_PDESCR;
+					consumed_chars = 0;
+				}
+				else if (!strncmp (parser->priv->context->next_token_start, "nullok:", 7)) {
+					parser->priv->context->next_token_start += 7;
+					retval = getToken (parser);
+					parser->priv->context->token_type = L_PNULLOK;
+					consumed_chars = 0;
+				}
+				else {
+					for (i=1; z[i] && (! isspace (z[i])) && 
+						     (z[i] != parser->priv->context->delimiter) && (z[i] != '*') && 
+						     (z[i] != '\'') && (z[i] != '"'); i++){}
+					parser->priv->context->token_type = L_RAWSTRING;
+					consumed_chars = i;
+				}
 			}
 		}
 	}
 
+	/* fallback for the token type */
+	if (parser->priv->context->token_type == G_MININT)
+		parser->priv->context->token_type = L_ILLEGAL;
+
 	if (!retval)
 		retval = token_as_string (parser->priv->context->next_token_start, consumed_chars);
-
  tok_end:
 	parser->priv->context->last_token_start = parser->priv->context->next_token_start;
 	parser->priv->context->next_token_start += consumed_chars;
@@ -1512,6 +1537,9 @@ fetch_forward (GdaSqlParser *parser, gint *out_nb_pushed, ...)
 			v1 = v2;
 		}
 		if (ttype != exp_type) {
+			g_value_reset (v1);
+			g_free (v1);
+
 			/* not what was expected => pop all the contexts */
 			for (; npushed > nmatched ; npushed--)
 				pop_tokenizer_context (parser);
