@@ -32,6 +32,15 @@
  * Sql expression
  *
  */
+
+/**
+ * gda_sql_expr_new
+ * @parent: a #GdaSqlStatementInsert, #GdaSqlStatementUpdate, #GdaSqlSelectField, #GdaSqlSelectTarget, #GdaSqlOperation
+ * 
+ * Creates a new #GdaSqlField structure, using @parent as its parent part.
+ *
+ * Returns: a new #GdaSqlField structure.
+ */
 GdaSqlExpr *
 gda_sql_expr_new (GdaSqlAnyPart *parent)
 {
@@ -42,6 +51,13 @@ gda_sql_expr_new (GdaSqlAnyPart *parent)
 	return expr;
 }
 
+/**
+ * gda_sql_expr_free
+ * @expr: a #GdaSqlExpr to be freed.
+ * 
+ * Frees a #GdaSqlExpr structure and its members.
+ *
+ */
 void
 gda_sql_expr_free (GdaSqlExpr *expr)
 {
@@ -69,6 +85,14 @@ gda_sql_expr_free (GdaSqlExpr *expr)
 	g_free (expr);
 }
 
+/**
+ * gda_sql_expr_copy
+ * @expr: a #GdaSqlExpr
+ * 
+ * Creates a new #GdaSqlExpr structure initated with the values stored in @expr.
+ *
+ * Returns: a new #GdaSqlExpr structure.
+ */
 GdaSqlExpr *
 gda_sql_expr_copy (GdaSqlExpr *expr)
 {
@@ -110,6 +134,15 @@ gda_sql_expr_copy (GdaSqlExpr *expr)
 	return copy;
 }
 
+/**
+ * gda_sql_expr_serialize
+ * @expr: a #GdaSqlExpr structure
+ * 
+ * Creates a new string representing a field. You need to free the returned string
+ * using g_free();
+ *
+ * Returns: a new string with the name of the field or "null" in case @expr is invalid.
+ */
 gchar *
 gda_sql_expr_serialize (GdaSqlExpr *expr)
 {
@@ -174,6 +207,15 @@ gda_sql_expr_serialize (GdaSqlExpr *expr)
 	return str;
 }
 
+/**
+ * gda_sql_expr_take_select
+ * @expr: a #GdaSqlExpr structure
+ * @stmt: a #GdaSqlStatement holding the #GdaSqlStatementSelect to take from
+ * 
+ * Sets the expression's parent to the #GdaSqlStatementSelect holded by @stmt. After
+ * calling this function @stmt is freed.
+ *
+ */
 void
 gda_sql_expr_take_select (GdaSqlExpr *expr, GdaSqlStatement *stmt)
 {
@@ -187,8 +229,13 @@ gda_sql_expr_take_select (GdaSqlExpr *expr, GdaSqlStatement *stmt)
 	}
 }
 
-/*
- * Any Table's field
+/**
+ * gda_sql_field_new
+ * @parent: a #GdaSqlStatementSelect, #GdaSqlStatementInsert, #GdaSqlStatementDelete, #GdaSqlStatementUpdate
+ * 
+ * Creates a new #GdaSqlField structure, using @parent as its parent part.
+ *
+ * Returns: a new #GdaSqlField structure.
  */
 GdaSqlField *
 gda_sql_field_new (GdaSqlAnyPart *parent)
@@ -200,6 +247,13 @@ gda_sql_field_new (GdaSqlAnyPart *parent)
 	return field;
 }
 
+/**
+ * gda_sql_field_free
+ * @field: a #GdaSqlField to be freed.
+ * 
+ * Frees a #GdaSqlField structure and its members.
+ *
+ */
 void
 gda_sql_field_free (GdaSqlField *field)
 {
@@ -210,6 +264,14 @@ gda_sql_field_free (GdaSqlField *field)
 	g_free (field);
 }
 
+/**
+ * gda_sql_field_copy
+ * @field: a #GdaSqlAnyPart
+ * 
+ * Creates a new GdaSqlField structure initated with the values stored in @field.
+ *
+ * Returns: a new #GdaSqlField structure.
+ */
 GdaSqlField *
 gda_sql_field_copy (GdaSqlField *field)
 {
@@ -219,10 +281,20 @@ gda_sql_field_copy (GdaSqlField *field)
 	copy = gda_sql_field_new (NULL);
 	if (field->field_name)
 		copy->field_name = g_strdup (field->field_name);
+	copy->validity_meta_table_column = field->validity_meta_table_column;
 
 	return copy;
 }
 
+/**
+ * gda_sql_field_serialize
+ * @field: a #GdaSqlField structure
+ * 
+ * Creates a new string representing a field. You need to free the returned string
+ * using g_free();
+ *
+ * Returns: a new string with the name of the field or "null" in case @field is invalid.
+ */
 gchar *
 gda_sql_field_serialize (GdaSqlField *field)
 {
@@ -232,18 +304,31 @@ gda_sql_field_serialize (GdaSqlField *field)
 		return _json_quote_string (field->field_name);
 }
 
+/**
+ * gda_sql_field_take_name
+ * @field: a #GdaSqlField structure
+ * @value: a #GValue holding a string to take from
+ * 
+ * Sets the field's name using the string holded by @value. When call, @value is freed using
+ * #gda_value_free().
+ *
+ */
 void
 gda_sql_field_take_name (GdaSqlField *field, GValue *value)
 {
 	if (value) {
 		field->field_name = g_value_dup_string (value);
-		g_value_reset (value);
-		g_free (value);
+		gda_value_free (value);
 	}
 }
 
-/*
- * Structure to hold one table
+/**
+ * gda_sql_table_new
+ * @parent: a #GdaSqlStatementSelect, #GdaSqlStatementInsert, #GdaSqlStatementDelete, #GdaSqlStatementUpdate
+ * 
+ * Creates a new #GdaSqlTable structure, using @parent as its parent part.
+ *
+ * Returns: a new #GdaSqlTable structure.
  */
 GdaSqlTable *
 gda_sql_table_new (GdaSqlAnyPart *parent)
@@ -255,6 +340,12 @@ gda_sql_table_new (GdaSqlAnyPart *parent)
 	return table;
 }
 
+/**
+ * gda_sql_table_free
+ * @table: a #GdaSqlTable structure to be freed
+ * 
+ * Frees a #GdaSqlTable structure and its members.
+ */
 void
 gda_sql_table_free (GdaSqlTable *table)
 {
@@ -265,6 +356,14 @@ gda_sql_table_free (GdaSqlTable *table)
 	g_free (table);
 }
 
+/**
+ * gda_sql_table_copy
+ * @table: a #GdaSqlTable structure to be copied
+ * 
+ * Creates a new #GdaSqlTable structure initated with the values stored in @table.
+ *
+ * Returns: a new #GdaSqlTable structure.
+ */
 GdaSqlTable *
 gda_sql_table_copy (GdaSqlTable *table)
 {
@@ -274,10 +373,20 @@ gda_sql_table_copy (GdaSqlTable *table)
 	copy = gda_sql_table_new (NULL);
 	if (table->table_name)
 		copy->table_name = g_strdup (table->table_name);
+	copy->validity_meta_object = table->validity_meta_object;
 
 	return copy;
 }
 
+/**
+ * gda_sql_table_serialize
+ * @field: a #GdaSqlTable structure
+ * 
+ * Creates a new string representing a table. You need to free the returned string
+ * using g_free();
+ *
+ * Returns: a new string with the name of the field or "null" in case @table is invalid.
+ */
 gchar *
 gda_sql_table_serialize (GdaSqlTable *table)
 {
@@ -287,21 +396,32 @@ gda_sql_table_serialize (GdaSqlTable *table)
 		return _json_quote_string (table->table_name);
 }
 
+/**
+ * gda_sql_table_take_name
+ * @field: a #GdaSqlTable structure
+ * @value: a #GValue holding a string to take from
+ * 
+ * Sets the table's name using the string holded by @value. When call, @value is freed using
+ * #gda_value_free().
+ *
+ */
 void
 gda_sql_table_take_name (GdaSqlTable *table, GValue *value)
 {
 	if (value) {
 		table->table_name = g_value_dup_string (value);
-		g_value_reset (value);
-		g_free (value);
+		gda_value_free (value);
 	}
 }
 
-
-/*
- * A function with any number of arguments
+/**
+ * gda_sql_function_new
+ * @parent: a #GdaSqlExpr structure
+ * 
+ * Creates a new #GdaSqlFunction structure initated.
+ *
+ * Returns: a new #GdaSqlFunction structure.
  */
-
 GdaSqlFunction *
 gda_sql_function_new (GdaSqlAnyPart *parent)
 {
@@ -312,6 +432,12 @@ gda_sql_function_new (GdaSqlAnyPart *parent)
 	return function;
 }
 
+/**
+ * gda_sql_function_free
+ * @function: a #GdaSqlFunction structure to be freed
+ * 
+ * Frees a #GdaSqlFunction structure and its members.
+ */
 void
 gda_sql_function_free (GdaSqlFunction *function)
 {
@@ -326,6 +452,14 @@ gda_sql_function_free (GdaSqlFunction *function)
 	g_free (function);
 }
 
+/**
+ * gda_sql_function_copy
+ * @function: a #GdaSqlFunction structure to be copied
+ * 
+ * Creates a new #GdaSqlFunction structure initated with the values stored in @function.
+ *
+ * Returns: a new #GdaSqlFunction structure.
+ */
 GdaSqlFunction *
 gda_sql_function_copy (GdaSqlFunction *function)
 {
@@ -345,11 +479,21 @@ gda_sql_function_copy (GdaSqlFunction *function)
 			gda_sql_any_part_set_parent (copy->args_list->data, copy);
 		}
 		copy->args_list = g_slist_reverse (copy->args_list);
-	}	
+	}
+	copy->validity_meta_function = function->validity_meta_function;
 
 	return copy;
 }
 
+/**
+ * gda_sql_function_serialize
+ * @function: a #GdaSqlFunction structure
+ * 
+ * Creates a new string representing a function. You need to free the returned string
+ * using g_free();
+ *
+ * Returns: a new string with the description of the function or "null" in case @function is invalid.
+ */
 gchar *
 gda_sql_function_serialize (GdaSqlFunction *function)
 {	
@@ -388,16 +532,33 @@ gda_sql_function_serialize (GdaSqlFunction *function)
 	}
 }
 
+/**
+ * gda_sql_function_take_name
+ * @function: a #GdaSqlFunction structure
+ * @value: a #GValue holding a string to take from
+ * 
+ * Sets the function's name using the string holded by @value. When call, @value is freed using
+ * #gda_value_free().
+ *
+ */
 void
 gda_sql_function_take_name (GdaSqlFunction *function, GValue *value)
 {
 	if (value) {
 		function->function_name = g_value_dup_string (value);
-		g_value_reset (value);
-		g_free (value);
+		gda_value_free (value);
 	}
 }
 
+/**
+ * gda_sql_function_take_name
+ * @function: a #GdaSqlFunction structure
+ * @args: a #GSList to take from
+ * 
+ * Sets the function's arguments to point to @args, then sets the
+ * list's data elements' parent to @function.
+ *
+ */
 void gda_sql_function_take_args_list (GdaSqlFunction *function, GSList *args)
 {
 	GSList *list;
@@ -407,8 +568,13 @@ void gda_sql_function_take_args_list (GdaSqlFunction *function, GSList *args)
 		gda_sql_any_part_set_parent (list->data, function);
 }
 
-/*
- * An operation
+/**
+ * gda_sql_operation_new
+ * @parent: a #GdaSqlExpr structure
+ * 
+ * Creates a new #GdaSqlOperation structure and sets its parent to @parent.
+ *
+ * Returns: a new #GdaSqlOperation structure.
  */
 GdaSqlOperation *
 gda_sql_operation_new (GdaSqlAnyPart *parent)
@@ -420,6 +586,12 @@ gda_sql_operation_new (GdaSqlAnyPart *parent)
 	return operation;
 }
 
+/**
+ * gda_sql_operation_free
+ * @operation: a #GdaSqlOperation structure to be freed
+ * 
+ * Frees a #GdaSqlOperation structure and its members.
+ */
 void
 gda_sql_operation_free (GdaSqlOperation *operation)
 {
@@ -432,6 +604,14 @@ gda_sql_operation_free (GdaSqlOperation *operation)
 	g_free (operation);
 }
 
+/**
+ * gda_sql_operation_copy
+ * @operation: a #GdaSqlOperation structure to be copied
+ * 
+ * Creates a new #GdaSqlOperation structure initated with the values stored in @operation.
+ *
+ * Returns: a new #GdaSqlOperation structure.
+ */
 GdaSqlOperation *
 gda_sql_operation_copy (GdaSqlOperation *operation)
 {
@@ -453,6 +633,15 @@ gda_sql_operation_copy (GdaSqlOperation *operation)
 	return copy;
 }
 
+/**
+ * gda_sql_operation_serialize
+ * @operation: a #GdaSqlOperation structure
+ * 
+ * Creates a new string representing an operator. You need to free the returned string
+ * using g_free();
+ *
+ * Returns: a new string with the description of the operator or "null" in case @operation is invalid.
+ */
 gchar *
 gda_sql_operation_serialize (GdaSqlOperation *operation)
 {	
@@ -489,69 +678,78 @@ gda_sql_operation_serialize (GdaSqlOperation *operation)
 	}
 }
 
+/**
+ * gda_sql_operation_operator_to_string
+ * @op: a #GdaSqlOperation structure
+ * 
+ * Returns a constant string representing a operator name. You don't need to free 
+ * the returned string.
+ *
+ * Returns: a string with the operator's name or NULL in case @op is invalid.
+ */
 const gchar *
-gda_sql_operation_operator_to_string (GdaSqlOperator op)
+gda_sql_operation_operator_to_string (GdaSqlOperatorType op)
 {
 	switch (op) {
-	case GDA_SQL_OPERATOR_AND:
+	case GDA_SQL_OPERATOR_TYPE_AND:
 		return "AND";
-	case GDA_SQL_OPERATOR_OR:
+	case GDA_SQL_OPERATOR_TYPE_OR:
 		return "OR";
-	case GDA_SQL_OPERATOR_NOT:
+	case GDA_SQL_OPERATOR_TYPE_NOT:
 		return "NOT";
-	case GDA_SQL_OPERATOR_EQ:
+	case GDA_SQL_OPERATOR_TYPE_EQ:
 		return "=";
-	case GDA_SQL_OPERATOR_IS:
+	case GDA_SQL_OPERATOR_TYPE_IS:
 		return "IS";
-	case GDA_SQL_OPERATOR_ISNULL:
+	case GDA_SQL_OPERATOR_TYPE_ISNULL:
 		return "IS NULL";
-	case GDA_SQL_OPERATOR_ISNOTNULL:
+	case GDA_SQL_OPERATOR_TYPE_ISNOTNULL:
 		return "IS NOT NULL";
-	case GDA_SQL_OPERATOR_IN:
+	case GDA_SQL_OPERATOR_TYPE_IN:
 		return "IN";
-	case GDA_SQL_OPERATOR_NOTIN:
+	case GDA_SQL_OPERATOR_TYPE_NOTIN:
 		return "NOT IN";
-	case GDA_SQL_OPERATOR_LIKE:
+	case GDA_SQL_OPERATOR_TYPE_LIKE:
 		return "LIKE";
-	case GDA_SQL_OPERATOR_BETWEEN:
+	case GDA_SQL_OPERATOR_TYPE_BETWEEN:
 		return "BETWEEN";
-	case GDA_SQL_OPERATOR_GT:
+	case GDA_SQL_OPERATOR_TYPE_GT:
 		return ">";
-	case GDA_SQL_OPERATOR_LT:
+	case GDA_SQL_OPERATOR_TYPE_LT:
 		return "<";
-	case GDA_SQL_OPERATOR_GEQ:
+	case GDA_SQL_OPERATOR_TYPE_GEQ:
 		return ">=";
-	case GDA_SQL_OPERATOR_LEQ:
+	case GDA_SQL_OPERATOR_TYPE_LEQ:
 		return "<=";
-	case GDA_SQL_OPERATOR_DIFF:
+	case GDA_SQL_OPERATOR_TYPE_DIFF:
 		return "!=";
-	case GDA_SQL_OPERATOR_REGEXP:
+	case GDA_SQL_OPERATOR_TYPE_REGEXP:
 		return "RE";
-	case GDA_SQL_OPERATOR_REGEXP_CI:
+	case GDA_SQL_OPERATOR_TYPE_REGEXP_CI:
 		return "CI_RE";
-	case GDA_SQL_OPERATOR_NOT_REGEXP:
+	case GDA_SQL_OPERATOR_TYPE_NOT_REGEXP:
 		return "!RE";
-	case GDA_SQL_OPERATOR_NOT_REGEXP_CI:
+	case GDA_SQL_OPERATOR_TYPE_NOT_REGEXP_CI:
 		return "!CI_RE";
-	case GDA_SQL_OPERATOR_SIMILAR:
+	case GDA_SQL_OPERATOR_TYPE_SIMILAR:
 		return "SIMILAR TO";
-	case GDA_SQL_OPERATOR_CONCAT:
+	case GDA_SQL_OPERATOR_TYPE_CONCAT:
 		return "||";
-	case GDA_SQL_OPERATOR_PLUS:
+	case GDA_SQL_OPERATOR_TYPE_PLUS:
 		return "+";
-	case GDA_SQL_OPERATOR_MINUS:
+	case GDA_SQL_OPERATOR_TYPE_MINUS:
 		return "-";
-	case GDA_SQL_OPERATOR_STAR:
+	case GDA_SQL_OPERATOR_TYPE_STAR:
 		return "*";
-	case GDA_SQL_OPERATOR_DIV:
+	case GDA_SQL_OPERATOR_TYPE_DIV:
 		return "/";
-	case GDA_SQL_OPERATOR_REM:
+	case GDA_SQL_OPERATOR_TYPE_REM:
 		return "%";
-	case GDA_SQL_OPERATOR_BITAND:
+	case GDA_SQL_OPERATOR_TYPE_BITAND:
 		return "&";
-	case GDA_SQL_OPERATOR_BITOR:
+	case GDA_SQL_OPERATOR_TYPE_BITOR:
 		return "|";
-	case GDA_SQL_OPERATOR_BITNOT:
+	case GDA_SQL_OPERATOR_TYPE_BITNOT:
 		return "~";
 	default:
 		g_error ("Unhandled operator constant %d\n", op);
@@ -559,77 +757,90 @@ gda_sql_operation_operator_to_string (GdaSqlOperator op)
 	}
 }
 
-GdaSqlOperator 
+/**
+ * gda_sql_operator_from_string
+ * @op: a #GdaSqlOperation structure
+ * 
+ * Returns #GdaSqlOperatorType that correspond with the string @op.
+ *
+ * Returns: #GdaSqlOperatorType
+ */
+GdaSqlOperatorType 
 gda_sql_operation_operator_from_string (const gchar *op)
 {
 	switch (g_ascii_toupper (*op)) {
 	case 'A':
-		return GDA_SQL_OPERATOR_AND;
+		return GDA_SQL_OPERATOR_TYPE_AND;
 	case 'O':
-		return GDA_SQL_OPERATOR_OR;
+		return GDA_SQL_OPERATOR_TYPE_OR;
 	case 'N':
-		return GDA_SQL_OPERATOR_NOT;
+		return GDA_SQL_OPERATOR_TYPE_NOT;
 	case '=':
-		return GDA_SQL_OPERATOR_EQ;
+		return GDA_SQL_OPERATOR_TYPE_EQ;
 	case 'I':
 		if (op[1] == 'S')
-			return GDA_SQL_OPERATOR_IS;
+			return GDA_SQL_OPERATOR_TYPE_IS;
 		else if (op[1] == 'N')
-			return GDA_SQL_OPERATOR_IN;
+			return GDA_SQL_OPERATOR_TYPE_IN;
 		break;
 	case 'L':
-		return GDA_SQL_OPERATOR_LIKE;
+		return GDA_SQL_OPERATOR_TYPE_LIKE;
 	case 'B':
-		return GDA_SQL_OPERATOR_BETWEEN;
+		return GDA_SQL_OPERATOR_TYPE_BETWEEN;
 	case '>':
 		if (op[1] == '=')
-			return GDA_SQL_OPERATOR_GEQ;
+			return GDA_SQL_OPERATOR_TYPE_GEQ;
 		else if (op[1] == 0)
-			return GDA_SQL_OPERATOR_GT;
+			return GDA_SQL_OPERATOR_TYPE_GT;
 		break;
 	case '<':
 		if (op[1] == '=')
-			return GDA_SQL_OPERATOR_LEQ;
+			return GDA_SQL_OPERATOR_TYPE_LEQ;
 		else if (op[1] == 0)
-			return GDA_SQL_OPERATOR_LT;
+			return GDA_SQL_OPERATOR_TYPE_LT;
 		break;
 	case '!':
 		if (op[1] == '=')
-			return GDA_SQL_OPERATOR_DIFF;
+			return GDA_SQL_OPERATOR_TYPE_DIFF;
 		else if (op[1] == 'R') 
-			return GDA_SQL_OPERATOR_NOT_REGEXP;
+			return GDA_SQL_OPERATOR_TYPE_NOT_REGEXP;
 		else
-			return GDA_SQL_OPERATOR_NOT_REGEXP_CI;
+			return GDA_SQL_OPERATOR_TYPE_NOT_REGEXP_CI;
 	case 'R':
-		return GDA_SQL_OPERATOR_REGEXP;
+		return GDA_SQL_OPERATOR_TYPE_REGEXP;
 	case 'C':
-		return GDA_SQL_OPERATOR_REGEXP_CI;
+		return GDA_SQL_OPERATOR_TYPE_REGEXP_CI;
 	case 'S':
-		return GDA_SQL_OPERATOR_SIMILAR;
+		return GDA_SQL_OPERATOR_TYPE_SIMILAR;
 	case '|':
 		if (op[1] == '|')
-			return GDA_SQL_OPERATOR_CONCAT;
+			return GDA_SQL_OPERATOR_TYPE_CONCAT;
 		else
-			return GDA_SQL_OPERATOR_BITOR;
+			return GDA_SQL_OPERATOR_TYPE_BITOR;
 	case '+':
-		return GDA_SQL_OPERATOR_PLUS;
+		return GDA_SQL_OPERATOR_TYPE_PLUS;
 	case '-':
-		return GDA_SQL_OPERATOR_MINUS;
+		return GDA_SQL_OPERATOR_TYPE_MINUS;
 	case '*':
-		return GDA_SQL_OPERATOR_STAR;
+		return GDA_SQL_OPERATOR_TYPE_STAR;
 	case '/':
-		return GDA_SQL_OPERATOR_DIV;
+		return GDA_SQL_OPERATOR_TYPE_DIV;
 	case '%':
-		return GDA_SQL_OPERATOR_REM;
+		return GDA_SQL_OPERATOR_TYPE_REM;
 	case '&':
-		return GDA_SQL_OPERATOR_BITAND;
+		return GDA_SQL_OPERATOR_TYPE_BITAND;
 	}
 	g_error ("Unhandled operator named '%s'\n", op);
 	return 0;
 }
 
-/*
- * A CASE expression
+/**
+ * gda_sql_case_new
+ * @parent: a #GdaSqlExpr structure
+ * 
+ * Creates a new #GdaSqlCase structure and sets its parent to @parent.
+ *
+ * Returns: a new #GdaSqlCase structure.
  */
 GdaSqlCase *
 gda_sql_case_new (GdaSqlAnyPart *parent)
@@ -641,6 +852,12 @@ gda_sql_case_new (GdaSqlAnyPart *parent)
 	return sc;
 }
 
+/**
+ * gda_sql_case_free
+ * @sc: a #GdaSqlCase structure to be freed
+ * 
+ * Frees a #GdaSqlCase structure and its members.
+ */
 void
 gda_sql_case_free (GdaSqlCase *sc)
 {
@@ -659,6 +876,14 @@ gda_sql_case_free (GdaSqlCase *sc)
 	g_free (sc);
 }
 
+/**
+ * gda_sql_case_copy
+ * @sc: a #GdaSqlCase structure to be copied
+ * 
+ * Creates a new #GdaSqlCase structure initated with the values stored in @sc.
+ *
+ * Returns: a new #GdaSqlCase structure.
+ */
 GdaSqlCase *
 gda_sql_case_copy (GdaSqlCase *sc)
 {
@@ -690,6 +915,15 @@ gda_sql_case_copy (GdaSqlCase *sc)
 	return copy;
 }
 
+/**
+ * gda_sql_case_serialize
+ * @sc: a #GdaSqlCase structure
+ * 
+ * Creates a new string representing a CASE clausure. You need to free the returned string
+ * using g_free();
+ *
+ * Returns: a new string with the description of the CASE clausure or "null" in case @sc is invalid.
+ */
 gchar *
 gda_sql_case_serialize (GdaSqlCase *sc)
 {	
@@ -738,8 +972,14 @@ gda_sql_case_serialize (GdaSqlCase *sc)
 	}
 }
 
-/*
- * Any expression in a SELECT ... before the FROM
+/**
+ * gda_sql_select_field_new
+ * @parent: a #GdaSqlStatementSelect structure
+ * 
+ * Creates a new #GdaSqlSelectField structure and sets its parent to @parent. A
+ * #GdaSqlSelectField is any expression in SELECT statements before the FROM clausure.
+ *
+ * Returns: a new #GdaSqlSelectField structure.
  */
 GdaSqlSelectField *
 gda_sql_select_field_new (GdaSqlAnyPart *parent)
@@ -751,6 +991,12 @@ gda_sql_select_field_new (GdaSqlAnyPart *parent)
 	return field;
 }
 
+/**
+ * gda_sql_select_field_free
+ * @field: a #GdaSqlSelectField structure to be freed
+ * 
+ * Frees a #GdaSqlSelectField structure and its members.
+ */
 void
 gda_sql_select_field_free (GdaSqlSelectField *field)
 {
@@ -765,6 +1011,14 @@ gda_sql_select_field_free (GdaSqlSelectField *field)
 	g_free (field);
 }
 
+/**
+ * gda_sql_select_field_copy
+ * @field: a #GdaSqlSelectField structure to be copied
+ * 
+ * Creates a new #GdaSqlSelectField structure initated with the values stored in @field.
+ *
+ * Returns: a new #GdaSqlSelectField structure.
+ */
 GdaSqlSelectField *
 gda_sql_select_field_copy (GdaSqlSelectField *field)
 {
@@ -782,9 +1036,22 @@ gda_sql_select_field_copy (GdaSqlSelectField *field)
 		copy->table_name = g_strdup (field->table_name);
 	if (field->as)
 		copy->as = g_strdup (field->as);
+
+	copy->validity_meta_object = field->validity_meta_object;
+	copy->validity_meta_table_column = field->validity_meta_table_column;
+
 	return copy;
 }
 
+/**
+ * gda_sql_select_field_serialize
+ * @field: a #GdaSqlSelectField structure
+ * 
+ * Creates a new string representing an expresion used as field in a SELECT statement
+ * before the FROM clausure.
+ *
+ * Returns: a new string with the description of the expression or "null" in case @field is invalid.
+ */
 gchar *
 gda_sql_select_field_serialize (GdaSqlSelectField *field)
 {	
@@ -827,32 +1094,33 @@ gda_sql_select_field_serialize (GdaSqlSelectField *field)
 	}
 }
 
+/**
+ * gda_sql_select_field_take_expr
+ * @field: a #GdaSqlSelectField structure
+ * @expr: a #GdaSqlExpr to take from
+ * 
+ * Sets the expression field in the #GdaSqlSelectField structure to point to @expr
+ * and modify it to sets its parent to @field.
+ */
 void
 gda_sql_select_field_take_expr (GdaSqlSelectField *field, GdaSqlExpr *expr)
 {
 	field->expr = expr;
 	gda_sql_any_part_set_parent (field->expr, field);
 
-	if (expr && expr->value) {
-		const gchar *str;
-		gchar *ptr;
-
-		str = g_value_get_string (expr->value);
-		if (_string_is_identifier (str)) {
-			for (ptr = (gchar *) str; *ptr; ptr++);
-			for (; (ptr > str) && (*ptr != '.'); ptr --);
-			if (ptr != str) {
-				field->field_name = g_strdup (ptr + 1);
-				*ptr = 0;
-				field->table_name = g_strdup (str);
-				*ptr = '.';
-			}
-			else 
-				field->field_name = g_strdup (ptr);
-		}
-	}
+	if (expr && expr->value) 
+		_split_identifier_string (g_value_dup_string (expr->value), &(field->table_name), &(field->field_name));
 }
 
+/**
+ * gda_sql_select_field_take_star_value
+ * @field: a #GdaSqlSelectField structure
+ * @value: a #GValue to take from
+ * 
+ * Sets the expression field's value in the #GdaSqlSelectField structure to point to @value;
+ * after this @field is the owner of @value.
+ *
+ */
 void
 gda_sql_select_field_take_star_value (GdaSqlSelectField *field, GValue *value)
 {
@@ -862,18 +1130,32 @@ gda_sql_select_field_take_star_value (GdaSqlSelectField *field, GValue *value)
 	}
 }
 
+/**
+ * gda_sql_select_field_take_alias
+ * @field: a #GdaSqlSelectField structure
+ * @alias: a #GValue to take from
+ * 
+ * Sets the 'as' field's string in the #GdaSqlSelectField structure. @alias is freed
+ * after call this function.
+ *
+ */
 void
 gda_sql_select_field_take_alias (GdaSqlSelectField *field, GValue *alias)
 {
 	if (alias) {
 		field->as = g_value_dup_string (alias);
-		g_value_reset (alias);
-		g_free (alias);
+		gda_value_free (alias);
 	}
 }
 
-/*
- * Any TARGET ... in a SELECT statement
+/**
+ * gda_sql_select_target_new
+ * @parent: a #GdaSqlSelectFrom
+ * 
+ * Creates a new #GdaSqlSelectTarget structure and sets its parent to @parent. A
+ * #GdaSqlSelectTarget is the table in a SELECT statement.
+ *
+ * Returns: a new #GdaSqlSelectTarget structure.
  */
 GdaSqlSelectTarget *
 gda_sql_select_target_new (GdaSqlAnyPart *parent)
@@ -885,6 +1167,12 @@ gda_sql_select_target_new (GdaSqlAnyPart *parent)
 	return target;
 }
 
+/**
+ * gda_sql_select_target_free
+ * @target: a #GdaSqlSelectTarget structure to be freed
+ * 
+ * Frees a #GdaSqlSelectTarget structure and its members.
+ */
 void
 gda_sql_select_target_free (GdaSqlSelectTarget *target)
 {
@@ -898,6 +1186,14 @@ gda_sql_select_target_free (GdaSqlSelectTarget *target)
 	g_free (target);
 }
 
+/**
+ * gda_sql_select_target_copy
+ * @target: a #GdaSqlSelectTarget structure to be copied
+ * 
+ * Creates a new #GdaSqlSelectTarget structure initated with the values stored in @target.
+ *
+ * Returns: a new #GdaSqlSelectTarget structure.
+ */
 GdaSqlSelectTarget *
 gda_sql_select_target_copy (GdaSqlSelectTarget *target)
 {
@@ -913,9 +1209,21 @@ gda_sql_select_target_copy (GdaSqlSelectTarget *target)
 		copy->table_name = g_strdup (target->table_name);
 	if (target->as)
 		copy->as = g_strdup (target->as);
+
+	copy->validity_meta_object = target->validity_meta_object;
+
 	return copy;
 }
 
+/**
+ * gda_sql_select_target_serialize
+ * @target: a #GdaSqlSelectTarget structure
+ * 
+ * Creates a new string representing a target used in a SELECT statement
+ * after the FROM clausure.
+ *
+ * Returns: a new string with the description of the expression or "null" in case @field is invalid.
+ */
 gchar *
 gda_sql_select_target_serialize (GdaSqlSelectTarget *target)
 {	
@@ -952,6 +1260,15 @@ gda_sql_select_target_serialize (GdaSqlSelectTarget *target)
 	}
 }
 
+/**
+ * gda_sql_select_target_take_table_name
+ * @target: a #GdaSqlSelectTarget structure
+ * @value: a #GValue to take from
+ * 
+ * Sets the target's name using the string stored in @value and the expression
+ * to set its value to point to value; after call this function the target owns
+ * @value, then you must not free it.
+ */
 void
 gda_sql_select_target_take_table_name (GdaSqlSelectTarget *target, GValue *value)
 {
@@ -962,6 +1279,14 @@ gda_sql_select_target_take_table_name (GdaSqlSelectTarget *target, GValue *value
 	}
 }
 
+/**
+ * gda_sql_select_target_take_table_name
+ * @target: a #GdaSqlSelectTarget structure
+ * @stmt: a #GValue to take from
+ * 
+ * Sets the target to be a SELECT subquery setting target's expression to use
+ * @stmt; after call this function the target owns @stmt, then you must not free it.
+ */
 void
 gda_sql_select_target_take_select (GdaSqlSelectTarget *target, GdaSqlStatement *stmt)
 {
@@ -971,19 +1296,35 @@ gda_sql_select_target_take_select (GdaSqlSelectTarget *target, GdaSqlStatement *
 	}
 }
 
+/**
+ * gda_sql_select_target_take_table_alias
+ * @target: a #GdaSqlSelectTarget structure
+ * @alias: a #GValue holding the alias string to take from
+ * 
+ * Sets the target alias (AS) to the string holded by @alias; after call
+ * this function @alias is freed.
+ */
 void
 gda_sql_select_target_take_alias (GdaSqlSelectTarget *target, GValue *alias)
 {
 	if (alias) {
 		target->as = g_value_dup_string (alias);
-		g_value_reset (alias);
-		g_free (alias);
+		gda_value_free (alias);
 	}
 }
 
 /*
  * Any JOIN ... in a SELECT statement
  */
+
+/**
+ * gda_sql_select_join_new
+ * @parent: a #GdaSqlSelectFrom
+ * 
+ * Creates a new #GdaSqlSelectJoin structure and sets its parent to @parent.
+ *
+ * Returns: a new #GdaSqlSelectJoin structure
+ */
 GdaSqlSelectJoin *
 gda_sql_select_join_new (GdaSqlAnyPart *parent)
 {
@@ -994,6 +1335,12 @@ gda_sql_select_join_new (GdaSqlAnyPart *parent)
 	return join;
 }
 
+/**
+ * gda_sql_select_join_free
+ * @join: a #GdaSqlSelectJoin structure to be freed
+ * 
+ * Frees a #GdaSqlSelectJoin structure and its members.
+ */
 void
 gda_sql_select_join_free (GdaSqlSelectJoin *join)
 {
@@ -1006,6 +1353,14 @@ gda_sql_select_join_free (GdaSqlSelectJoin *join)
 	g_free (join);
 }
 
+/**
+ * gda_sql_select_join_copy
+ * @join: a #GdaSqlSelectJoin structure to be copied
+ * 
+ * Creates a new #GdaSqlSelectJoin structure initated with the values stored in @join.
+ *
+ * Returns: a new #GdaSqlSelectJoin structure.
+ */
 GdaSqlSelectJoin *
 gda_sql_select_join_copy (GdaSqlSelectJoin *join)
 {
@@ -1031,6 +1386,14 @@ gda_sql_select_join_copy (GdaSqlSelectJoin *join)
 	return copy;
 }
 
+/**
+ * gda_sql_select_join_type_to_string
+ * @type: a #GdaSqlSelectJoinType structure to be copied
+ * 
+ * Creates a new string representing the join type.
+ *
+ * Returns: a string representing the Join type.
+ */
 const gchar *
 gda_sql_select_join_type_to_string (GdaSqlSelectJoinType type)
 {
@@ -1053,6 +1416,14 @@ gda_sql_select_join_type_to_string (GdaSqlSelectJoinType type)
 	}
 }
 
+/**
+ * gda_sql_select_join_serialize
+ * @join: a #GdaSqlSelectJoin structure
+ * 
+ * Creates a new string description of the join used in a SELECT statement.
+ *
+ * Returns: a new string with the description of the join or "null" in case @join is invalid.
+ */
 gchar *
 gda_sql_select_join_serialize (GdaSqlSelectJoin *join)
 {	
@@ -1105,6 +1476,15 @@ gda_sql_select_join_serialize (GdaSqlSelectJoin *join)
 /*
  * Any FROM ... in a SELECT statement
  */
+
+/**
+ * gda_sql_select_from_new
+ * @parent: a #GdaSqlStatementSelect
+ * 
+ * Creates a new #GdaSqlSelectFrom structure and sets its parent to @parent.
+ *
+ * Returns: a new #GdaSqlSelectFrom structure
+ */
 GdaSqlSelectFrom *
 gda_sql_select_from_new (GdaSqlAnyPart *parent)
 {
@@ -1115,6 +1495,12 @@ gda_sql_select_from_new (GdaSqlAnyPart *parent)
 	return from;
 }
 
+/**
+ * gda_sql_select_from_free
+ * @from: a #GdaSqlSelectFrom structure to be freed
+ * 
+ * Frees a #GdaSqlSelectFrom structure and its members.
+ */
 void
 gda_sql_select_from_free (GdaSqlSelectFrom *from)
 {
@@ -1132,6 +1518,14 @@ gda_sql_select_from_free (GdaSqlSelectFrom *from)
 	g_free (from);
 }
 
+/**
+ * gda_sql_select_from_copy
+ * @from: a #GdaSqlSelectFrom structure to be copied
+ * 
+ * Creates a new #GdaSqlSelectFrom structure initated with the values stored in @from.
+ *
+ * Returns: a new #GdaSqlSelectFrom structure.
+ */
 GdaSqlSelectFrom *
 gda_sql_select_from_copy (GdaSqlSelectFrom *from)
 {
@@ -1158,6 +1552,14 @@ gda_sql_select_from_copy (GdaSqlSelectFrom *from)
 	return copy;
 }
 
+/**
+ * gda_sql_select_from_serialize
+ * @from: a #GdaSqlSelectFrom structure
+ * 
+ * Creates a new string description of the FROM clausure used in a SELECT statement.
+ *
+ * Returns: a new string with the description of the FROM or "null" in case @from is invalid.
+ */
 gchar *
 gda_sql_select_from_serialize (GdaSqlSelectFrom *from)
 {	
@@ -1205,6 +1607,14 @@ gda_sql_select_from_serialize (GdaSqlSelectFrom *from)
 	}
 }
 
+/**
+ * gda_sql_select_from_take_new_target
+ * @from: a #GdaSqlSelectFrom structure
+ * @target: a #GdaSqlSelectTarget to take from
+ * 
+ * Append @target to the targets in the FROM clausure and set @target's parent to
+ * @from; after call this function @from owns @target then you must not free it.
+ */
 void
 gda_sql_select_from_take_new_target (GdaSqlSelectFrom *from, GdaSqlSelectTarget *target)
 {
@@ -1212,6 +1622,14 @@ gda_sql_select_from_take_new_target (GdaSqlSelectFrom *from, GdaSqlSelectTarget
 	gda_sql_any_part_set_parent (target, from);
 }
 
+/**
+ * gda_sql_select_from_take_new_join
+ * @from: a #GdaSqlSelectFrom structure
+ * @join: a #GdaSqlSelectJoin to take from
+ * 
+ * Append @join to the joins in the FROM clausure and set @join's parent to
+ * @from; after call this function @from owns @join then you must not free it.
+ */
 void
 gda_sql_select_from_take_new_join  (GdaSqlSelectFrom *from, GdaSqlSelectJoin *join)
 {
@@ -1222,6 +1640,15 @@ gda_sql_select_from_take_new_join  (GdaSqlSelectFrom *from, GdaSqlSelectJoin *jo
 /*
  * Any expression in a SELECT ... after the ORDER BY
  */
+
+/**
+ * gda_sql_select_order_new
+ * @parent: a #GdaSqlStatementSelect
+ * 
+ * Creates a new #GdaSqlSelectOrder structure and sets its parent to @parent.
+ *
+ * Returns: a new #GdaSqlSelectOrder structure
+ */
 GdaSqlSelectOrder *
 gda_sql_select_order_new (GdaSqlAnyPart *parent)
 {
@@ -1232,6 +1659,12 @@ gda_sql_select_order_new (GdaSqlAnyPart *parent)
 	return order;
 }
 
+/**
+ * gda_sql_select_order_free
+ * @order: a #GdaSqlSelectOrder structure to be freed
+ * 
+ * Frees a #GdaSqlSelectOrder structure and its members.
+ */
 void
 gda_sql_select_order_free (GdaSqlSelectOrder *order)
 {
@@ -1243,6 +1676,14 @@ gda_sql_select_order_free (GdaSqlSelectOrder *order)
 	g_free (order);
 }
 
+/**
+ * gda_sql_select_order_copy
+ * @order: a #GdaSqlSelectOrder structure to be copied
+ * 
+ * Creates a new #GdaSqlSelectOrder structure initated with the values stored in @order.
+ *
+ * Returns: a new #GdaSqlSelectOrder structure.
+ */
 GdaSqlSelectOrder *
 gda_sql_select_order_copy (GdaSqlSelectOrder *order)
 {
@@ -1261,6 +1702,14 @@ gda_sql_select_order_copy (GdaSqlSelectOrder *order)
 	return copy;
 }
 
+/**
+ * gda_sql_select_order_serialize
+ * @order: a #GdaSqlSelectOrder structure
+ * 
+ * Creates a new string description of the ORDER BY clausure used in a SELECT statement.
+ *
+ * Returns: a new string with the description of the ORDER BY or "null" in case @order is invalid.
+ */
 gchar *
 gda_sql_select_order_serialize (GdaSqlSelectOrder *order)
 {	
