@@ -25,6 +25,7 @@
 #include <sql-parser/gda-statement-struct.h>
 #include <sql-parser/gda-statement-struct-pspec.h>
 #include <sql-parser/gda-statement-struct-decl.h>
+#include <libgda/gda-meta-struct.h>
 
 typedef struct _GdaSqlExpr      GdaSqlExpr;
 typedef struct _GdaSqlField     GdaSqlField;
@@ -66,6 +67,9 @@ void             gda_sql_expr_take_select    (GdaSqlExpr *expr, GdaSqlStatement
 struct _GdaSqlField {
 	GdaSqlAnyPart       any;
 	gchar              *field_name;
+
+	/* validity check with a connection */
+	GdaMetaTableColumn *validity_meta_table_column;
 };
 GdaSqlField     *gda_sql_field_new            (GdaSqlAnyPart *parent);
 void             gda_sql_field_free           (GdaSqlField *field);
@@ -83,8 +87,8 @@ struct _GdaSqlTable
 	GdaSqlAnyPart       any;
 	gchar              *table_name;
 
-	/* GdaMetaStore check */
-	gpointer            full_table_name;
+	/* validity check with a connection */
+	GdaMetaDbObject    *validity_meta_object;
 };
 
 GdaSqlTable     *gda_sql_table_new            (GdaSqlAnyPart *parent);
@@ -103,8 +107,8 @@ struct _GdaSqlFunction {
 	gchar              *function_name;
 	GSList             *args_list;
 
-	/* GdaDict check */
-	gchar              *full_function_name;
+	/* validity check with a connection */
+	gpointer            validity_meta_function; /* to be replaced with a pointer to a structure representing a DBMS data type in GdaMetaStruct */
 };
 
 GdaSqlFunction  *gda_sql_function_new            (GdaSqlAnyPart *parent);
@@ -120,52 +124,52 @@ void             gda_sql_function_take_args_list (GdaSqlFunction *function, GSLi
  * An operation on one or more expressions
  */
 typedef enum {
-	GDA_SQL_OPERATOR_AND,
-	GDA_SQL_OPERATOR_OR,
+	GDA_SQL_OPERATOR_TYPE_AND,
+	GDA_SQL_OPERATOR_TYPE_OR,
 
-	GDA_SQL_OPERATOR_EQ, 
-	GDA_SQL_OPERATOR_IS, 
-	GDA_SQL_OPERATOR_LIKE,
-	GDA_SQL_OPERATOR_BETWEEN,
-	GDA_SQL_OPERATOR_GT,
-	GDA_SQL_OPERATOR_LT,
-	GDA_SQL_OPERATOR_GEQ,
-	GDA_SQL_OPERATOR_LEQ,
-	GDA_SQL_OPERATOR_DIFF,
-	GDA_SQL_OPERATOR_REGEXP,
-	GDA_SQL_OPERATOR_REGEXP_CI,
-	GDA_SQL_OPERATOR_NOT_REGEXP,
-	GDA_SQL_OPERATOR_NOT_REGEXP_CI,
-	GDA_SQL_OPERATOR_SIMILAR,
-	GDA_SQL_OPERATOR_ISNULL,
-	GDA_SQL_OPERATOR_ISNOTNULL,
-	GDA_SQL_OPERATOR_NOT,
-	GDA_SQL_OPERATOR_IN,
-	GDA_SQL_OPERATOR_NOTIN,
+	GDA_SQL_OPERATOR_TYPE_EQ, 
+	GDA_SQL_OPERATOR_TYPE_IS, 
+	GDA_SQL_OPERATOR_TYPE_LIKE,
+	GDA_SQL_OPERATOR_TYPE_BETWEEN,
+	GDA_SQL_OPERATOR_TYPE_GT,
+	GDA_SQL_OPERATOR_TYPE_LT,
+	GDA_SQL_OPERATOR_TYPE_GEQ,
+	GDA_SQL_OPERATOR_TYPE_LEQ,
+	GDA_SQL_OPERATOR_TYPE_DIFF,
+	GDA_SQL_OPERATOR_TYPE_REGEXP,
+	GDA_SQL_OPERATOR_TYPE_REGEXP_CI,
+	GDA_SQL_OPERATOR_TYPE_NOT_REGEXP,
+	GDA_SQL_OPERATOR_TYPE_NOT_REGEXP_CI,
+	GDA_SQL_OPERATOR_TYPE_SIMILAR,
+	GDA_SQL_OPERATOR_TYPE_ISNULL,
+	GDA_SQL_OPERATOR_TYPE_ISNOTNULL,
+	GDA_SQL_OPERATOR_TYPE_NOT,
+	GDA_SQL_OPERATOR_TYPE_IN,
+	GDA_SQL_OPERATOR_TYPE_NOTIN,
 
-	GDA_SQL_OPERATOR_CONCAT,
-	GDA_SQL_OPERATOR_PLUS,
-	GDA_SQL_OPERATOR_MINUS,
-	GDA_SQL_OPERATOR_STAR,
-	GDA_SQL_OPERATOR_DIV,
-	GDA_SQL_OPERATOR_REM,
-	GDA_SQL_OPERATOR_BITAND,
-	GDA_SQL_OPERATOR_BITOR,
-	GDA_SQL_OPERATOR_BITNOT
-} GdaSqlOperator;
+	GDA_SQL_OPERATOR_TYPE_CONCAT,
+	GDA_SQL_OPERATOR_TYPE_PLUS,
+	GDA_SQL_OPERATOR_TYPE_MINUS,
+	GDA_SQL_OPERATOR_TYPE_STAR,
+	GDA_SQL_OPERATOR_TYPE_DIV,
+	GDA_SQL_OPERATOR_TYPE_REM,
+	GDA_SQL_OPERATOR_TYPE_BITAND,
+	GDA_SQL_OPERATOR_TYPE_BITOR,
+	GDA_SQL_OPERATOR_TYPE_BITNOT
+} GdaSqlOperatorType;
 
 struct _GdaSqlOperation {
-	GdaSqlAnyPart    any;
-	GdaSqlOperator   operator;
-	GSList          *operands;
+	GdaSqlAnyPart       any;
+	GdaSqlOperatorType  operator;
+	GSList             *operands;
 };
 
 GdaSqlOperation  *gda_sql_operation_new            (GdaSqlAnyPart *parent);
 void              gda_sql_operation_free           (GdaSqlOperation *operation);
 GdaSqlOperation  *gda_sql_operation_copy           (GdaSqlOperation *operation);
 gchar            *gda_sql_operation_serialize      (GdaSqlOperation *operation);
-const gchar      *gda_sql_operation_operator_to_string (GdaSqlOperator op);
-GdaSqlOperator    gda_sql_operation_operator_from_string (const gchar *op);
+const gchar      *gda_sql_operation_operator_to_string (GdaSqlOperatorType op);
+GdaSqlOperatorType    gda_sql_operation_operator_from_string (const gchar *op);
 
 /*
  * A CASE expression
@@ -185,7 +189,7 @@ GdaSqlCase        *gda_sql_case_copy           (GdaSqlCase *scase);
 gchar             *gda_sql_case_serialize      (GdaSqlCase *scase);
 
 /*
- * Any expression in a SELECT ... before the FROM
+ * Any expression in a SELECT ... before the FROM clause
  */
 struct _GdaSqlSelectField
 {
@@ -195,8 +199,9 @@ struct _GdaSqlSelectField
 	gchar              *table_name; /* may be NULL if expr does not refer to a table.field */
 	gchar              *as; 
 
-	/* GdaDict check */
-	gchar              *full_table_name;
+	/* validity check with a connection */
+	GdaMetaDbObject    *validity_meta_object;
+	GdaMetaTableColumn *validity_meta_table_column;
 };
 
 GdaSqlSelectField *gda_sql_select_field_new            (GdaSqlAnyPart *parent);
@@ -219,8 +224,8 @@ struct _GdaSqlSelectTarget
 	gchar              *table_name; /* may be NULL if expr does not refer to a table */
 	gchar              *as; 
 
-	/* GdaDict check */
-	gchar              *full_table_name;
+	/* validity check with a connection */
+	GdaMetaDbObject    *validity_meta_object;
 };
 
 GdaSqlSelectTarget *gda_sql_select_target_new            (GdaSqlAnyPart *parent);
