@@ -96,8 +96,11 @@ static GdaSet       *pragma_set;
 void
 _gda_sqlite_provider_meta_init (GdaServerProvider *provider)
 {
+	static GStaticMutex init_mutex = G_STATIC_MUTEX_INIT;
 	InternalStatementItem i;
 
+	g_static_mutex_lock (&init_mutex);
+
         internal_parser = gda_server_provider_internal_get_parser (provider);
 	internal_params = gda_set_new (NULL);
 
@@ -124,6 +127,8 @@ _gda_sqlite_provider_meta_init (GdaServerProvider *provider)
 
 	pragma_set = gda_set_new_inline (2, "tblname", G_TYPE_STRING, "",
 					 "idxname", G_TYPE_STRING, "");
+
+	g_static_mutex_unlock (&init_mutex);
 }
 
 gboolean
@@ -196,10 +201,11 @@ _gda_sqlite_meta__btypes (GdaServerProvider *prov, GdaConnection *cnc,
 			break;
 		}
 	}
-	gda_value_free (vint);
-	if (retval)
+	if (retval) 
 		retval = gda_meta_store_modify (store, context->table_name, mod_model, NULL, error, NULL);
 	g_object_unref (mod_model);
+	gda_value_free (vint);
+
 	return retval;
 }
 
@@ -299,6 +305,15 @@ _gda_sqlite_meta__el_types (GdaServerProvider *prov, GdaConnection *cnc,
 	return TRUE;
 }
 
+gboolean
+_gda_sqlite_meta_el_types (GdaServerProvider *prov, GdaConnection *cnc, 
+			   GdaMetaStore *store, GdaMetaContext *context, GError **error,
+			   const GValue *specific_name)
+{
+	/* feature not supported by SQLite */
+	return TRUE;
+}
+
 gboolean
 _gda_sqlite_meta__collations (GdaServerProvider *prov, GdaConnection *cnc, 
 			      GdaMetaStore *store, GdaMetaContext *context, GError **error)
@@ -451,20 +466,18 @@ fill_tables_views_model (GdaConnection *cnc,
                         if (*(g_value_get_string (tvalue)) == 'v')
                                 is_view = TRUE;
                         g_value_set_boolean ((v1 = gda_value_new (G_TYPE_BOOLEAN)), TRUE);
-                        if (strcmp (schema_name, "main")) {
-                                str = g_strdup_printf ("%s.%s", schema_name, g_value_get_string (cvalue));
-                                g_value_take_string ((v2 = gda_value_new (G_TYPE_STRING)), str);
-                        }
+			str = g_strdup_printf ("%s.%s", schema_name, g_value_get_string (cvalue));
+			g_value_take_string ((v2 = gda_value_new (G_TYPE_STRING)), str);
                         if (! append_a_row (to_tables_model, error, 9,
-                                            FALSE, catalog_value,
-                                            FALSE, p_table_schema,
-                                            FALSE, cvalue,
-                                            FALSE, is_view ? view_type_value : table_type_value,
-                                            TRUE, v1,
-                                            FALSE, NULL,
-                                            FALSE, cvalue,
-                                            v2 ? TRUE : FALSE, v2 ? v2 : cvalue,
-                                            FALSE, NULL))
+                                            FALSE, catalog_value, /* table_catalog */
+                                            FALSE, p_table_schema, /* table_schema */
+                                            FALSE, cvalue, /* table_name */
+                                            FALSE, is_view ? view_type_value : table_type_value, /* table_type */
+                                            TRUE, v1, /* is_insertable_into */
+                                            FALSE, NULL, /* table_comments */
+                                            FALSE, cvalue, /* table_short_name */
+                                            TRUE, v2, /* table_full_name */
+                                            FALSE, NULL)) /* table_owner */
                                 retval = FALSE;
                         if (is_view && ! append_a_row (to_views_model, error, 6,
                                                        FALSE, catalog_value,
@@ -646,7 +659,7 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 			g_value_set_string ((v6 = gda_value_new (G_TYPE_STRING)), "string");
 		else
 			g_value_set_string ((v6 = gda_value_new (G_TYPE_STRING)), g_type_name (gtype));
-		if (! append_a_row (mod_model, error, 25, 
+		if (! append_a_row (mod_model, error, 24, 
 				    FALSE, catalog_value, /* table_catalog */
 				    FALSE, p_table_schema, /* table_schema */
 				    FALSE, p_table_name, /* table_name */
@@ -655,8 +668,7 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 				    FALSE, gda_data_model_get_value_at (tmpmodel, 4, i), /* column default */
 				    TRUE, v3, /* is_nullable */
 				    TRUE, v2, /* data_type */
-				    FALSE, zero_value, /* array_dimension */
-				    FALSE, NULL, /* element_type */
+				    FALSE, NULL, /* array_spec */
 				    TRUE, v6, /* gtype */
 				    FALSE, NULL, /* character_maximum_length */
 				    FALSE, NULL, /* character_octet_length */
