@@ -120,6 +120,7 @@ gda_vconnection_hub_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		if (type == 0) {
 			static GTypeInfo info = {
 				sizeof (GdaVconnectionHubClass),
@@ -132,7 +133,10 @@ gda_vconnection_hub_get_type (void)
 				(GInstanceInitFunc) gda_vconnection_hub_init
 			};
 			
+		g_static_mutex_lock (&registering);
+		if (type == 0)
 			type = g_type_register_static (GDA_TYPE_VCONNECTION_DATA_MODEL, "GdaVconnectionHub", &info, 0);
+		g_static_mutex_unlock (&registering);
 		}
 	}
 
