@@ -97,6 +97,7 @@ gda_virtual_connection_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		if (type == 0) {
 			static GTypeInfo info = {
 				sizeof (GdaVirtualConnectionClass),
@@ -109,7 +110,10 @@ gda_virtual_connection_get_type (void)
 				(GInstanceInitFunc) gda_virtual_connection_init
 			};
 			
+		g_static_mutex_lock (&registering);
+		if (type == 0)
 			type = g_type_register_static (PARENT_TYPE, "GdaVirtualConnection", &info, G_TYPE_FLAG_ABSTRACT);
+		g_static_mutex_unlock (&registering);
 		}
 	}
 
