@@ -33,7 +33,7 @@
 #include <libgda/gda-data-access-wrapper.h>
 #include <libgda/gda-blob-op.h>
 #include "../gda-sqlite.h"
-
+#include <sql-parser/gda-statement-struct-util.h>
 
 struct _GdaVproviderDataModelPrivate {
 	
@@ -123,6 +123,7 @@ gda_vprovider_data_model_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		if (type == 0) {
 			static GTypeInfo info = {
 				sizeof (GdaVproviderDataModelClass),
@@ -135,7 +136,10 @@ gda_vprovider_data_model_get_type (void)
 				(GInstanceInitFunc) gda_vprovider_data_model_init
 			};
 			
+		g_static_mutex_lock (&registering);
+		if (type == 0)
 			type = g_type_register_static (GDA_TYPE_VIRTUAL_PROVIDER, "GdaVproviderDataModel", &info, 0);
+		g_static_mutex_unlock (&registering);
 		}
 	}
 
@@ -408,6 +412,7 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 	for (i = 0; i < ncols; i++) {
 		GdaColumn *column;
 		const gchar *name, *type;
+		gchar *newcolname;
 
 		if (i != 0)
 			g_string_append (sql, ", ");
@@ -420,15 +425,16 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 			g_string_free (sql, TRUE);
 			return SQLITE_ERROR;
 		}
+
 		name = gda_column_get_name (column);
-		type = gda_column_get_dbms_type (column);
-		if (!type) 
-			type = g_type_name (gda_column_get_g_type (column));
-		if (!name) {
-			*pzErr = sqlite3_mprintf (_("Can't get data model's column name for column %d"), i);
-			g_string_free (sql, TRUE);
-			return SQLITE_ERROR;
-		}
+		if (!name || !(*name))
+			newcolname = g_strdup_printf ("_%d", i + 1);
+		else if (_identifier_needs_quotes (name))
+			newcolname = g_strdup_printf ("\"%s\"", name);
+		else
+			newcolname = g_strdup (name);
+
+		type = g_type_name (gda_column_get_g_type (column));
 		if (!type) {
 			*pzErr = sqlite3_mprintf (_("Can't get data model's column type or type for column %d"), i);
 			g_string_free (sql, TRUE);
@@ -461,7 +467,8 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 				type = "text";
 		}
 
-		g_string_append (sql, name);
+		g_string_append (sql, newcolname);
+		g_free (newcolname);
 		g_string_append_c (sql, ' ');
 		g_string_append (sql, type);
 		if (! gda_column_get_allow_null (column)) 
