@@ -112,6 +112,7 @@ gda_capi_recordset_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaCapiRecordsetClass),
 			(GBaseInitFunc) NULL,
@@ -123,7 +124,10 @@ gda_capi_recordset_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_capi_recordset_init
 		};
-		type = g_type_register_static (GDA_TYPE_PMODEL, "GdaCapiRecordset", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (GDA_TYPE_PMODEL, "GdaCapiRecordset", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 
 	return type;
