@@ -4,13 +4,9 @@
 #include <libgda/libgda.h>
 #include <stdarg.h>
 
-#ifdef HAVE_CHECK
-#include <check.h>
-#else
 #define fail(x) g_warning (x)
 #define fail_if(x,y) if (x) g_warning (y)
 #define fail_unless(x,y) if (!(x)) g_warning (y)
-#endif
 
 #define CHECK_EXTRA_INFO
 /*#undef CHECK_EXTRA_INFO*/
@@ -50,209 +46,13 @@ static gboolean check_data_model_remove_row (GdaDataModel *model, gint row);
 static GList *make_values_list (gint dummy, ...);
 static void free_values_list (GList *list);
 
-#ifdef HAVE_CHECK
-
-START_TEST (test1)
-{
-	prepend_null_row = FALSE;
-	defer_sync = FALSE;
-	if (!do_test_read_direct_1 ()) 
-		fail ("test read only with no sample size failed (defer sync = FALSE)");
-	defer_sync = TRUE;
-	if (!do_test_read_direct_1 ()) 
-		fail ("test read only with no sample size failed (defer sync = TRUE)");
-}
-END_TEST
-
-START_TEST (test2)
-{
-	prepend_null_row = FALSE;
-	defer_sync = FALSE;
-	if (!do_test_read_direct_2 ()) 
-		fail ("test read only with default sample size failed (defer sync = FALSE)");
-	defer_sync = TRUE;
-	if (!do_test_read_direct_2 ()) 
-		fail ("test read only with default sample size failed (defer sync = TRUE)");
-}
-END_TEST
-
-START_TEST (test3)
-{
-	prepend_null_row = FALSE;
-	defer_sync = FALSE;
-	if (!do_test_array ()) 
-		fail ("test read write failed (defer sync = FALSE)");
-	defer_sync = TRUE;
-	if (!do_test_array ()) 
-		fail ("test read write failed (defer sync = TRUE)");
-}
-END_TEST
-
-START_TEST (test4)
-{
-	prepend_null_row = TRUE;
-	defer_sync = FALSE;
-	if (!do_test_read_direct_1 ()) 
-		fail ("test read only with no sample size failed (defer sync = FALSE)");
-	defer_sync = TRUE;
-	if (!do_test_read_direct_1 ()) 
-		fail ("test read only with no sample size failed (defer sync = TRUE)");
-}
-END_TEST
-
-START_TEST (test5)
-{
-	prepend_null_row = TRUE;
-	defer_sync = FALSE;
-	if (!do_test_read_direct_2 ()) 
-		fail ("test read only with default sample size failed (defer sync = FALSE)");
-	defer_sync = TRUE;
-	if (!do_test_read_direct_2 ()) 
-		fail ("test read only with default sample size failed (defer sync = TRUE)");
-}
-END_TEST
-
-START_TEST (test6)
-{
-	prepend_null_row = TRUE;
-	defer_sync = FALSE;
-	if (!do_test_array ()) 
-		fail ("test read write failed (defer sync = FALSE)");
-	defer_sync = TRUE;
-	if (!do_test_array ()) 
-		fail ("test read write failed (defer sync = TRUE)");
-}
-END_TEST
-
-START_TEST (test7)
-{
-	prepend_null_row = FALSE;
-	defer_sync = FALSE;
-	if (!do_test_prop_change ())
-		fail ("test prop change failed");
-	defer_sync = TRUE;
-	if (!do_test_prop_change ())
-		fail ("test prop change failed");
-}
-END_TEST
-
-START_TEST (test8)
-{
-	prepend_null_row = FALSE;
-	defer_sync = FALSE;
-	if (!do_test_proxied_model_modif ())
-		fail ("proxied model modif failed");
-	defer_sync = TRUE;
-	if (!do_test_proxied_model_modif ())
-		fail ("proxied model modif failed");
-
-	prepend_null_row = TRUE;
-	defer_sync = FALSE;
-	if (!do_test_proxied_model_modif ())
-		fail ("poxied model mod failed");
-	defer_sync = TRUE;
-	if (!do_test_proxied_model_modif ())
-		fail ("poxied model mod failed");
-}
-END_TEST
-
-Suite *
-test_1_suite (void)
-{
-	Suite *s = suite_create ("Test");
-
-	/* Core test case */
-        TCase *tc_core = tcase_create ("Test read only");
-	tcase_set_timeout (tc_core, 10);
-	tcase_add_test (tc_core, test1);
-	tcase_add_test (tc_core, test2);
-	suite_add_tcase (s, tc_core);
-
-	tc_core = tcase_create ("Test read write");
-	tcase_set_timeout (tc_core, 10);
-	tcase_add_test (tc_core, test3);
-	suite_add_tcase (s, tc_core);
-	
-	return s;
-}
-
-Suite *
-test_2_suite (void)
-{
-	Suite *s = suite_create ("Test");
-
-	/* Core test case */
-        TCase *tc_core = tcase_create ("Test read only");
-	tcase_set_timeout (tc_core, 10);
-	tcase_add_test (tc_core, test4);
-	tcase_add_test (tc_core, test5);
-	suite_add_tcase (s, tc_core);
-
-	tc_core = tcase_create ("Test read write");
-	tcase_set_timeout (tc_core, 10);
-	tcase_add_test (tc_core, test6);
-	suite_add_tcase (s, tc_core);
-
-	tc_core = tcase_create ("Test prop change");
-	tcase_set_timeout (tc_core, 10);
-	tcase_add_test (tc_core, test7);
-	suite_add_tcase (s, tc_core);
-	
-	return s;
-}
-
-Suite *
-test_3_suite (void)
-{
-	Suite *s = suite_create ("Test");
-
-	/* Core test case */
-        TCase *tc_core = tcase_create ("Proxied model modified");
-	tcase_set_timeout (tc_core, 10);
-	tcase_add_test (tc_core, test8);
-	suite_add_tcase (s, tc_core);
-	
-	return s;
-}
 
 int
 main (int argc, char **argv)
 {
 	int number_failed = 0;
 
-	gda_init ("check-data-proxy", PACKAGE_VERSION, argc, argv);
-
-	Suite *s = test_1_suite ();
-	SRunner *sr = srunner_create (s);
-	srunner_run_all (sr, CK_NORMAL);
-	number_failed += srunner_ntests_failed (sr);
-	srunner_free (sr);
-
-	s = test_2_suite ();
-	sr = srunner_create (s);
-	srunner_run_all (sr, CK_NORMAL);
-	number_failed += srunner_ntests_failed (sr);
-	srunner_free (sr);
-
-	s = test_3_suite ();
-	sr = srunner_create (s);
-	srunner_run_all (sr, CK_NORMAL);
-	number_failed += srunner_ntests_failed (sr);
-	srunner_free (sr);
-	
-	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-}
-#else
-
-/*
- * The Check library is not available
- */
-int
-main (int argc, char **argv)
-{
-	int number_failed = 0;
-
-	gda_init ("check-data-proxy", PACKAGE_VERSION, argc, argv);
+	gda_init ();
 
 	prepend_null_row = FALSE;
 	defer_sync  = FALSE;
@@ -323,9 +123,12 @@ main (int argc, char **argv)
 	if (!do_test_proxied_model_modif ())
 		number_failed ++;
 
+	if (number_failed == 0)
+		g_print ("Ok.\n");
+	else
+		g_print ("%d failed\n", number_failed);
 	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
-#endif
 
 
 static gboolean
@@ -335,10 +138,10 @@ do_test_read_direct_1 ()
 	gchar *file;
 	GdaDataModel *import, *proxy;
 	GSList *errors;
-	GdaParameterList *options;
+	GdaSet *options;
 
 	file = g_build_filename (CHECK_FILES, "tests", "data-models", FILE, NULL);
-	options = gda_parameter_list_new_inline (NULL, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE, NULL);
+	options = gda_set_new_inline (1, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE);
 	import = gda_data_model_import_new_file (file, TRUE, options);
 	g_free (file);
 	g_object_unref (options);
@@ -351,8 +154,7 @@ do_test_read_direct_1 ()
 		return FALSE;
 	}
 
-	proxy = g_object_new (GDA_TYPE_DATA_PROXY, "dict", gda_object_get_dict (GDA_OBJECT (import)), 
-			      "model", import, "sample_size", 0, NULL);
+	proxy = g_object_new (GDA_TYPE_DATA_PROXY, "model", import, "sample_size", 0, NULL);
 	g_object_unref (import);
 	if (!proxy) {
 #ifdef CHECK_EXTRA_INFO
@@ -371,10 +173,10 @@ do_test_read_direct_2 ()
 	gchar *file;
 	GdaDataModel *import, *proxy;
 	GSList *errors;
-	GdaParameterList *options;
+	GdaSet *options;
 
 	file = g_build_filename (CHECK_FILES, "tests", "data-models", FILE, NULL);
-	options = gda_parameter_list_new_inline (NULL, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE, NULL);
+	options = gda_set_new_inline (1, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE);
 	import = gda_data_model_import_new_file (file, TRUE, options);
 	g_free (file);
 	g_object_unref (options);
@@ -407,10 +209,10 @@ do_test_array ()
 	gchar *file;
 	GdaDataModel *import, *model, *proxy;
 	GSList *errors;
-	GdaParameterList *options;
+	GdaSet *options;
 
 	file = g_build_filename (CHECK_FILES, "tests", "data-models", FILE, NULL);
-	options = gda_parameter_list_new_inline (NULL, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE, NULL);
+	options = gda_set_new_inline (1, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE);
 	import = gda_data_model_import_new_file (file, TRUE, options);
 	g_free (file);
 	g_object_unref (options);
@@ -423,7 +225,7 @@ do_test_array ()
 		return FALSE;
 	}
 
-	model = gda_data_model_array_copy_model (import, &error);
+	model = (GdaDataModel*) gda_data_model_array_copy_model (import, &error);
 	if (!model) {
 #ifdef CHECK_EXTRA_INFO
 		g_print ("ERROR: Could not copy GdaDataModelImport into a GdaDataModelArray: %s\n", 
@@ -454,10 +256,10 @@ do_test_prop_change (void)
 	gchar *file;
 	GdaDataModel *import, *model, *proxy;
 	GSList *errors;
-	GdaParameterList *options;
+	GdaSet *options;
 
 	file = g_build_filename (CHECK_FILES, "tests", "data-models", FILE, NULL);
-	options = gda_parameter_list_new_inline (NULL, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE, NULL);
+	options = gda_set_new_inline (1, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE);
 	import = gda_data_model_import_new_file (file, TRUE, options);
 	g_free (file);
 	g_object_unref (options);
@@ -470,7 +272,7 @@ do_test_prop_change (void)
 		return FALSE;
 	}
 
-	model = gda_data_model_array_copy_model (import, &error);
+	model = (GdaDataModel*) gda_data_model_array_copy_model (import, &error);
 	if (!model) {
 #ifdef CHECK_EXTRA_INFO
 		g_print ("ERROR: Could not copy GdaDataModelImport into a GdaDataModelArray: %s\n", 
@@ -540,7 +342,6 @@ do_test_prop_change (void)
 	clean_expected_signals (proxy);
 
 	retval = TRUE;
- out:
 	g_object_unref (proxy);
 	return retval;
 }
@@ -553,10 +354,10 @@ do_test_proxied_model_modif (void)
 	gchar *file;
 	GdaDataModel *import, *model, *proxy;
 	GSList *errors;
-	GdaParameterList *options;
+	GdaSet *options;
 
 	file = g_build_filename (CHECK_FILES, "tests", "data-models", FILE, NULL);
-	options = gda_parameter_list_new_inline (NULL, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE, NULL);
+	options = gda_set_new_inline (1, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE);
 	import = gda_data_model_import_new_file (file, TRUE, options);
 	g_free (file);
 	g_object_unref (options);
@@ -569,7 +370,7 @@ do_test_proxied_model_modif (void)
 		return FALSE;
 	}
 
-	model = gda_data_model_array_copy_model (import, &error);
+	model = (GdaDataModel*) gda_data_model_array_copy_model (import, &error);
 	if (!model) {
 #ifdef CHECK_EXTRA_INFO
 		g_print ("ERROR: Could not copy GdaDataModelImport into a GdaDataModelArray: %s\n", 
