@@ -39,15 +39,17 @@
 #ifndef G_OS_WIN32
 #include <signal.h>
 #include <pwd.h>
+#else
+#include <stdlib.h>
+#include <windows.h>
+#endif
+
+#ifdef HAVE_READLINE
+#include <readline/readline.h>
 #endif
 
 /* options */
-gchar *pass = NULL;
-gchar *user = NULL;
-
-gchar *dsn = NULL;
-gchar *direct = NULL;
-gchar *prov = NULL;
+gboolean ask_pass = FALSE;
 
 gchar *single_command = NULL;
 gchar *commandsfile = NULL;
@@ -57,13 +59,8 @@ gboolean list_providers = FALSE;
 
 gchar *outfile = NULL;
 
-
 static GOptionEntry entries[] = {
-        { "cnc", 'c', 0, G_OPTION_ARG_STRING, &direct, "Direct connection string", "connection string"},
-        { "provider", 'p', 0, G_OPTION_ARG_STRING, &prov, "Provider name", "provider"},
-        { "dsn", 's', 0, G_OPTION_ARG_STRING, &dsn, "Data source", "DSN"},
-        { "user", 'U', 0, G_OPTION_ARG_STRING, &user, "Username", "username" },
-        { "password", 'P', 0, G_OPTION_ARG_STRING, &pass, "Password", "password" },
+        { "no-password-ask", 'p', 0, G_OPTION_ARG_NONE, &ask_pass, "Don't ast for a password when it is empty", NULL },
 
         { "output-file", 'o', 0, G_OPTION_ARG_STRING, &outfile, "Output file", "output file"},
         { "command", 'C', 0, G_OPTION_ARG_STRING, &single_command, "Run only single command (SQL or internal) and exit", "command" },
@@ -121,14 +118,14 @@ MainData *main_data;
 GString *prompt = NULL;
 
 static gchar   *read_a_line (MainData *data);
+static char   **completion_func (const char *text, int start, int end);
 static void     compute_prompt (MainData *data, GString *string, gboolean in_command);
 static gboolean set_output_file (MainData *data, const gchar *file, GError **error);
 static gboolean set_input_file (MainData *data, const gchar *file, GError **error);
 static void     output_data_model (MainData *data, GdaDataModel *model);
 static void     output_string (MainData *data, const gchar *str);
-static ConnectionSetting *open_connection (MainData *data, const gchar *cnc_name, const gchar *dsn,
-					   const gchar *provider, const gchar *direct, 
-					   const gchar *user, const gchar *pass, GError **error);
+static ConnectionSetting *open_connection (MainData *data, const gchar *cnc_name, const gchar *cnc_string,
+					   GError **error);
 static GdaDataModel *list_all_dsn (MainData *data);
 static GdaDataModel *list_all_providers (MainData *data);
 
@@ -148,7 +145,7 @@ main (int argc, char *argv[])
 	GSList *list;
 	prompt = g_string_new ("");
 
-	context = g_option_context_new (_("Gda SQL console"));        
+	context = g_option_context_new (_("[DSN|connection string]..."));        
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
         if (!g_option_context_parse (context, &argc, &argv, &error)) {
                 g_print ("Can't parse arguments: %s\n", error->message);
@@ -156,18 +153,11 @@ main (int argc, char *argv[])
 		goto cleanup;
         }
         g_option_context_free (context);
-        gda_init ("Gda SQL console", PACKAGE_VERSION, argc, argv);
+        gda_init ();
 	data = g_new0 (MainData, 1);
 	data->parameters = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 	main_data = data;
-
-	/* test for command line or env. variable connection string */
-	if (!direct) {
-		if (argc >= 2)
-			direct = argv [1];
-		else 
-			direct = getenv ("GDA_SQL_CNC");
-	}
+	ask_pass = !ask_pass;
 
 	/* output file */
 	if (outfile) {
@@ -208,15 +198,17 @@ main (int argc, char *argv[])
 			data->input_stream = stdin;
 	}
 
-	/* check connection parameters coherence */
-	if (direct && dsn) {
-                g_fprintf (stderr, _("DSN and connection string are exclusive\n"));
-		exit_status = EXIT_FAILURE;
-                goto cleanup;
-        }
-
 	/* welcome message */
 	if (!data->output_stream) {
+#ifdef G_OS_WIN32
+		HANDLE wHnd;
+		SMALL_RECT windowSize = {0, 0, 139, 49};
+
+		wHnd = GetStdHandle(STD_OUTPUT_HANDLE);
+
+		SetConsoleTitle ("Gda console, version " PACKAGE_VERSION);
+		SetConsoleWindowInfo (wHnd, TRUE, &windowSize);
+#endif
 		g_print (_("Welcome to the GDA SQL console, version " PACKAGE_VERSION));
 		g_print ("\n\n");
 		g_print (_("Type: .copyright to show usage and distribution terms\n"
@@ -226,12 +218,43 @@ main (int argc, char *argv[])
 			   "      or any query terminated by a semicolon\n\n"));
 	}
 
-        if (direct || dsn) {
+	/* open connections if specified */
+	gint i;
+	for (i = 1; i < argc; i++) {
 		/* open connection */
 		ConnectionSetting *cs;
-		cs = open_connection (data, NULL, dsn, prov, direct, user, pass, &error);
+		GdaDataSourceInfo *info = NULL;
+		gchar *str;
+
+                info = gda_config_get_dsn (argv[i]);
+		if (info)
+			str = g_strdup (info->name);
+		else
+			str = g_strdup_printf ("c%d", i-1);
+		if (!data->output_stream) 
+			g_print (_("Opening connection '%s' for: %s\n"), str, argv[i]);
+		cs = open_connection (data, str, argv[i], &error);
+		g_free (str);
 		if (!cs) {
-			g_print (_("Can't open connection: %s\n"), error && error->message ? error->message : _("No detail"));
+			g_print (_("Can't open connection %d: %s\n"), i,
+				 error && error->message ? error->message : _("No detail"));
+			exit_status = EXIT_FAILURE;
+			goto cleanup;
+		}
+	}
+	if (getenv ("GDA_SQL_CNC")) {
+		ConnectionSetting *cs;
+		gchar *str;
+		const gchar *envstr = getenv ("GDA_SQL_CNC");
+		str = g_strdup_printf ("c%d", i-1);
+		if (!data->output_stream) 
+			g_print (_("Opening connection '%s' for: %s (GDA_SQL_CNC environment variable)\n"), 
+				   str, envstr);
+		cs = open_connection (data, str, envstr, &error);
+		g_free (str);
+		if (!cs) {
+			g_print (_("Can't open connection defined by GDA_SQL_CNC: %s\n"),
+				 error && error->message ? error->message : _("No detail"));
 			exit_status = EXIT_FAILURE;
 			goto cleanup;
 		}
@@ -243,6 +266,7 @@ main (int argc, char *argv[])
 	/* loop over commands */
 	setup_sigint_handler ();
 	init_input ();
+	set_completion_func (completion_func);
 	init_history ();
 	for (;;) {
 		gchar *cmde;
@@ -369,7 +393,7 @@ display_result (MainData *data, GdaInternalCommandResult *res)
 				pnode = xmlNewNode (NULL, BAD_CAST "parameter");
 				xmlAddChild (node, pnode);
 				xmlSetProp (pnode, BAD_CAST "name", 
-					    gda_holder_get_id (GDA_HOLDER (list->data)));
+					    BAD_CAST gda_holder_get_id (GDA_HOLDER (list->data)));
 				value = gda_holder_get_value (GDA_HOLDER (list->data));
 				vnode = gda_value_to_xml (value);
 				xmlAddChild (pnode, vnode);
@@ -390,7 +414,7 @@ display_result (MainData *data, GdaInternalCommandResult *res)
 				pnode = xmlNewNode (NULL, BAD_CAST "li");
 				xmlAddChild (node, pnode);
 				xmlSetProp (pnode, BAD_CAST "name", 
-					    gda_holder_get_id (GDA_HOLDER (list->data)));
+					    BAD_CAST gda_holder_get_id (GDA_HOLDER (list->data)));
 				value = gda_holder_get_value (GDA_HOLDER (list->data));
 				vnode = gda_value_to_xml (value);
 				xmlAddChild (pnode, vnode);
@@ -432,7 +456,7 @@ display_result (MainData *data, GdaInternalCommandResult *res)
 		case OUTPUT_FORMAT_XML:
 			buffer = xmlBufferCreate ();
 			node = xmlNewNode (NULL, BAD_CAST "txt");
-			xmlNodeSetContent (node, res->u.txt->str);
+			xmlNodeSetContent (node, BAD_CAST res->u.txt->str);
 			xmlNodeDump (buffer, NULL, node, 0, 1);
 			output_string (data, (gchar *) xmlBufferContent (buffer));
 			xmlBufferFree (buffer);
@@ -441,7 +465,7 @@ display_result (MainData *data, GdaInternalCommandResult *res)
 		case OUTPUT_FORMAT_HTML: 
 			buffer = xmlBufferCreate ();
 			node = xmlNewNode (NULL, BAD_CAST "p");
-			xmlNodeSetContent (node, res->u.txt->str);
+			xmlNodeSetContent (node, BAD_CAST res->u.txt->str);
 			xmlNodeDump (buffer, NULL, node, 0, 1);
 			output_string (data, (gchar *) xmlBufferContent (buffer));
 			xmlBufferFree (buffer);
@@ -602,22 +626,42 @@ static GdaInternalCommandResult *
 execute_external_command (MainData *data, const gchar *command, GError **error)
 {
 	GdaInternalCommandResult *res = NULL;
+	GdaBatch *batch;
+	const GSList *stmt_list;
 	GdaStatement *stmt;
 	GdaSet *params;
 	GObject *obj;
 	const gchar *remain = NULL;
 
-	stmt = gda_sql_parser_parse_string (data->current->parser, command, &remain, error);
-	if (! stmt)
+	batch = gda_sql_parser_parse_string_as_batch (data->current->parser, command, &remain, error);
+	if (!batch)
 		return NULL;
 	if (remain) {
-		g_set_error (error, 0, 0,
-			     _("More than one SQL statement, remaining is: %s"), remain);
+		g_object_unref (batch);
+		return NULL;
+	}
+	
+	stmt_list = gda_batch_get_statements (batch);
+	if (!stmt_list) {
+		g_object_unref (batch);
 		return NULL;
 	}
 
-	if (!gda_statement_get_parameters (stmt, &params, error))
+	if (stmt_list->next) {
+		g_set_error (error, 0, 0,
+			     _("More than one SQL statement"));
+		g_object_unref (batch);
 		return NULL;
+	}
+		
+	stmt = GDA_STATEMENT (stmt_list->data);
+	g_object_ref (stmt);
+	g_object_unref (batch);
+
+	if (!gda_statement_get_parameters (stmt, &params, error)) {
+		g_object_unref (stmt);
+		return NULL;
+	}
 
 	/* fill parameters with some defined parameters */
 	if (params && params->holders) {
@@ -826,14 +870,11 @@ find_connection_from_name (MainData *data, const gchar *name)
  * Open a connection
  */
 static ConnectionSetting*
-open_connection (MainData *data, const gchar *cnc_name, const gchar *dsn, const gchar *provider, const gchar *direct, 
-		 const gchar *user, const gchar *pass,
-		 GError **error)
+open_connection (MainData *data, const gchar *cnc_name, const gchar *cnc_string, GError **error)
 {
 	GdaConnection *newcnc = NULL;
 	ConnectionSetting *cs = NULL;
 	static gint cncindex = 0;
-	gchar *auth_string = NULL;
 
 	if (cnc_name && ! connection_name_is_valid (cnc_name)) {
 		g_set_error (error, 0, 0,
@@ -841,27 +882,75 @@ open_connection (MainData *data, const gchar *cnc_name, const gchar *dsn, const
 		return NULL;
 	}
 
-	if (user) {
-		if (pass)
-			auth_string = g_strdup_printf ("USERNAME=%s;PASSWORD=%s", user, pass);
-		else
-			auth_string = g_strdup_printf ("USERNAME=%s", user);
+	GdaDataSourceInfo *info;
+	gchar *user, *pass, *real_cnc, *real_provider, *real_auth_string = NULL;
+	gda_connection_string_split (cnc_string, &real_cnc, &real_provider, &user, &pass);
+	if (!real_cnc) {
+		g_free (user);
+		g_free (pass);
+		g_free (real_provider);
+		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_DSN_NOT_FOUND_ERROR, 
+			     _("Malformed connection string '%s'"), cnc_string);
+		return NULL;
 	}
-	if (dsn) {
-                GdaDataSourceInfo *info = NULL;
-                info = gda_config_get_dsn (dsn);
-                if (!info) {
-			if (!direct)
-				g_set_error (error, 0, 0,
-					     _("DSN '%s' is not declared"), dsn);
+
+	if (ask_pass) {
+		if (user && !*user) {
+			gchar buf[80];
+			g_print (_("\tUsername for '%s': "), cnc_name);
+			if (scanf ("%80s", buf) == -1) {
+				g_free (real_cnc);
+				g_free (user);
+				g_free (pass);
+				g_free (real_provider);
+				g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_DSN_NOT_FOUND_ERROR, 
+					     _("No username for '%s'"), cnc_string);
+				return NULL;
+			}
+			g_free (user);
+			user = g_strdup (buf);
 		}
-                else 
-                        newcnc = gda_connection_open_from_dsn (info->name, auth_string ? auth_string : info->auth_string,
-							       0, error);
-        }
-        if (!newcnc && direct) 
-		newcnc = gda_connection_open_from_string (provider, direct, auth_string, 0, error);
-	g_free (auth_string);
+		if (pass && !*pass) {
+			gchar buf[80];
+			g_print (_("\tPassword for '%s': "), cnc_name);
+			if (scanf ("%80s", buf) == -1) {
+				g_free (real_cnc);
+				g_free (user);
+				g_free (pass);
+				g_free (real_provider);
+				g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_DSN_NOT_FOUND_ERROR, 
+					     _("No password for '%s'"), cnc_string);
+				return NULL;
+			}
+			g_free (pass);
+			pass = g_strdup (buf);
+		}
+		if (user || pass) {
+			gchar *s1;
+			s1 = gda_rfc1738_encode (user);
+			if (pass) {
+				gchar *s2;
+				s2 = gda_rfc1738_encode (pass);
+				real_auth_string = g_strdup_printf ("USERNAME=%s;PASSWORD=%s", s1, s2);
+				g_free (s2);
+			}
+			else
+				real_auth_string = g_strdup_printf ("USERNAME=%s", s1);
+			g_free (s1);
+		}
+	}
+	
+	info = gda_config_get_dsn (real_cnc);
+	if (info && !real_provider)
+		newcnc = gda_connection_open_from_dsn (cnc_string, real_auth_string, 0, error);
+	else 
+		newcnc = gda_connection_open_from_string (NULL, cnc_string, real_auth_string, 0, error);
+	
+	g_free (real_cnc);
+	g_free (user);
+	g_free (pass);
+	g_free (real_provider);
+	g_free (real_auth_string);
 
 	if (newcnc) {
 		cs = g_new0 (ConnectionSetting, 1);
@@ -876,16 +965,15 @@ open_connection (MainData *data, const gchar *cnc_name, const gchar *dsn, const
 		data->settings = g_slist_append (data->settings, cs);
 		data->current = cs;
 
-		/* set the GdaDict object's attributes */
 		GdaMetaStore *store;
 		gboolean update_store = FALSE;
 
-		if (dsn) {
+		if (info) {
 			gchar *filename;
 #define LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S ".libgda"
 			filename = g_strdup_printf ("%s%sgda-sql-%s.db", 
 						    g_get_home_dir (), LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S,
-						    dsn);
+						    info->name);
 			if (! g_file_test (filename, G_FILE_TEST_EXISTS))
 				update_store = TRUE;
 			store = gda_meta_store_new_with_file (filename);
@@ -900,7 +988,7 @@ open_connection (MainData *data, const gchar *cnc_name, const gchar *dsn, const
 		if (update_store) {
 			GError *lerror = NULL;
 			if (!data->output_stream) {
-				g_print (_("Getting database schema information, "
+				g_print (_("\tGetting database schema information, "
 					   "this may take some time... "));
 				fflush (stdout);
 			}
@@ -913,7 +1001,7 @@ open_connection (MainData *data, const gchar *cnc_name, const gchar *dsn, const
 			}
 			else
 				if (!data->output_stream) 
-					g_print (_(" Done.\n"));
+					g_print (_("Done.\n"));
 		}
 
 		g_object_unref (store);
@@ -980,7 +1068,7 @@ output_data_model (MainData *data, GdaDataModel *model)
 		for (j = 0; j < ncols; j++) {
 			const gchar *cstr;
 			cstr = gda_data_model_get_column_title (model, j);
-			col_node = xmlNewChild (row_node, NULL, BAD_CAST "th", cstr);
+			col_node = xmlNewChild (row_node, NULL, BAD_CAST "th", BAD_CAST cstr);
 			xmlSetProp (col_node, BAD_CAST "align", BAD_CAST "center");
 		}
 
@@ -991,17 +1079,14 @@ output_data_model (MainData *data, GdaDataModel *model)
 				const GValue *value;
 				value = gda_data_model_get_value_at (model, j, i);
 				str = gda_value_stringify (value);
-				col_node = xmlNewChild (row_node, NULL, BAD_CAST "td", str);
+				col_node = xmlNewChild (row_node, NULL, BAD_CAST "td", BAD_CAST str);
 				xmlSetProp (col_node, BAD_CAST "align", BAD_CAST "left");
 				g_free (str);
 			}
 		}
 
 		node = xmlNewChild (html, NULL, BAD_CAST "p", NULL);
-		if (nrows > 1)
-			str = g_strdup_printf (_("(%d rows)"), nrows);
-		else
-			str = g_strdup_printf (_("(%d row)"), nrows);
+		g_strdup_printf (ngettext ("(%d row)", "(%d rows)", nrows), nrows);
 		xmlNodeSetContent (node, BAD_CAST str);
 		g_free (str);
 
@@ -1240,7 +1325,7 @@ build_internal_commands_list (MainData *data)
 	c = g_new0 (GdaInternalCommand, 1);
 	c->group = _("General");
 	c->name = g_strdup_printf (_("%s [CNC_NAME [DSN [USER [PASSWORD]]]]"), "c");
-	c->description = _("Connect to another defined data source (DSN, see \\dsn_list)");
+	c->description = _("Connect to another defined data source (DSN, see \\l)");
 	c->args = NULL;
 	c->command_func = (GdaInternalCommandFunc) extra_command_manage_cnc;
 	c->user_data = data;
@@ -1252,7 +1337,7 @@ build_internal_commands_list (MainData *data)
 	c->name = g_strdup_printf (_("%s [CNC_NAME]"), "close");
 	c->description = _("Close a connection");
 	c->args = NULL;
-	c->command_func = (GdaInternalCommandFunc)extra_command_close_cnc;
+	c->command_func = (GdaInternalCommandFunc) extra_command_close_cnc;
 	c->user_data = data;
 	c->arguments_delimiter_func = NULL;
 	commands->commands = g_slist_prepend (commands->commands, c);
@@ -1269,7 +1354,7 @@ build_internal_commands_list (MainData *data)
 
 	c = g_new0 (GdaInternalCommand, 1);
 	c->group = _("General");
-	c->name = "dsn_list";
+	c->name = "l";
 	c->description = _("List configured data sources (DSN)");
 	c->args = NULL;
 	c->command_func = (GdaInternalCommandFunc) extra_command_list_dsn;
@@ -1580,9 +1665,8 @@ extra_command_qecho (GdaConnection *cnc, const gchar **args,
 	return res;
 }
 
-static 
-GdaInternalCommandResult *extra_command_list_dsn (GdaConnection *cnc, const gchar **args,
-						  GError **error, MainData *data)
+static GdaInternalCommandResult *
+extra_command_list_dsn (GdaConnection *cnc, const gchar **args, GError **error, MainData *data)
 {
 	GdaInternalCommandResult *res;
 	
@@ -1592,9 +1676,8 @@ GdaInternalCommandResult *extra_command_list_dsn (GdaConnection *cnc, const gcha
 	return res;
 }
 
-static 
-GdaInternalCommandResult *extra_command_list_providers (GdaConnection *cnc, const gchar **args,
-							GError **error, MainData *data)
+static GdaInternalCommandResult *
+extra_command_list_providers (GdaConnection *cnc, const gchar **args, GError **error, MainData *data)
 {
 	GdaInternalCommandResult *res;
 	
@@ -1613,8 +1696,8 @@ vconnection_hub_foreach_cb (GdaConnection *cnc, const gchar *ns, GString *string
 }
 
 static 
-GdaInternalCommandResult *extra_command_manage_cnc (GdaConnection *cnc, const gchar **args,
-						    GError **error, MainData *data)
+GdaInternalCommandResult *
+extra_command_manage_cnc (GdaConnection *cnc, const gchar **args, GError **error, MainData *data)
 {
 	/* arguments:
 	 * 0 = connection name
@@ -1640,7 +1723,7 @@ GdaInternalCommandResult *extra_command_manage_cnc (GdaConnection *cnc, const gc
 				if (args[3])
 					pass = args[3];
 			}
-			cs = open_connection (data, args[0], args[1], NULL, args[1], user, pass, error);
+			cs = open_connection (data, args[0], args[1], error);
 			if (cs) {
 				GdaInternalCommandResult *res;
 				
@@ -1654,6 +1737,7 @@ GdaInternalCommandResult *extra_command_manage_cnc (GdaConnection *cnc, const gc
 		else {
 			/* switch to another already opened connection */
 			ConnectionSetting *cs;
+
 			cs = find_connection_from_name (data, args[0]);
 			if (cs) {
 				GdaInternalCommandResult *res;
@@ -1664,6 +1748,76 @@ GdaInternalCommandResult *extra_command_manage_cnc (GdaConnection *cnc, const gc
 				return res;
 			}
 			else {
+				if (*args [0] == '~') {
+					if (*(args[0] + 1)) {
+						cs = find_connection_from_name (data, args[0] + 1);
+						if (!cs) {
+							g_set_error (error, 0, 0,
+								     _("No connection named '%s' found"), args[0] + 1);
+							return NULL;
+						}
+					}
+					else if (!data->current) {
+						g_set_error (error, 0, 0,
+							     _("No current connection"));
+						return NULL;
+					}
+					else {
+						if (* (data->current->name) == '~') 
+							cs = find_connection_from_name (data, data->current->name + 1);
+						if (!cs) {
+							gchar *tmp;
+							tmp = g_strdup_printf ("~%s", data->current->name);
+							cs = find_connection_from_name (data, tmp);
+							g_free (tmp);
+						}
+						if (cs) {
+							GdaInternalCommandResult *res;
+							
+							res = g_new0 (GdaInternalCommandResult, 1);
+							res->type = GDA_INTERNAL_COMMAND_RESULT_EMPTY;
+							data->current = cs;
+							return res;
+						}
+
+						cs = data->current;
+					}
+
+					ConnectionSetting *ncs = g_new0 (ConnectionSetting, 1);
+					GdaMetaStore *store;
+					GdaInternalCommandResult *res;
+					
+					ncs->name = g_strdup_printf ("~%s", cs->name);
+					store = gda_connection_get_meta_store (cs->cnc);
+					ncs->cnc = gda_meta_store_get_internal_connection (store);
+					g_object_ref (ncs->cnc);
+					ncs->parser = gda_connection_create_parser (ncs->cnc);
+					
+					data->settings = g_slist_append (data->settings, ncs);
+					data->current = ncs;
+
+					GError *lerror = NULL;
+					if (!data->output_stream) {
+						g_print (_("Getting database schema information, "
+							   "this may take some time... "));
+						fflush (stdout);
+					}
+					if (!gda_connection_update_meta_store (ncs->cnc, NULL, &lerror)) {
+						if (!data->output_stream) 
+							g_print (_("error: %s\n"), 
+								 lerror && lerror->message ? lerror->message : _("No detail"));
+						if (lerror)
+							g_error_free (lerror);
+					}
+					else
+						if (!data->output_stream) 
+							g_print (_("Done.\n"));
+					
+					res = g_new0 (GdaInternalCommandResult, 1);
+					res->type = GDA_INTERNAL_COMMAND_RESULT_EMPTY;
+					return res;
+				}
+
 				g_set_error (error, 0, 0,
 					     _("No connection named '%s' found"), args[0]);
 				return NULL;
@@ -1732,9 +1886,9 @@ GdaInternalCommandResult *extra_command_manage_cnc (GdaConnection *cnc, const gc
 			gda_data_model_set_value_at (model, 2, row, value, NULL);
 			gda_value_free (value);
 
-			/* only get USERNAME from the the authentification string */
+			/* only get USERNAME from the the authentication string */
 			GdaQuarkList* ql;
-			cstr = gda_connection_get_authentification (cs->cnc);
+			cstr = gda_connection_get_authentication (cs->cnc);
 			ql = gda_quark_list_new_from_string (cstr);
 			cstr = gda_quark_list_find (ql, "USERNAME");
 			value = gda_value_new_from_string (cstr ? cstr : "", G_TYPE_STRING);
@@ -1751,8 +1905,8 @@ GdaInternalCommandResult *extra_command_manage_cnc (GdaConnection *cnc, const gc
 }
 
 static 
-GdaInternalCommandResult *extra_command_close_cnc (GdaConnection *cnc, const gchar **args,
-						   GError **error, MainData *data)
+GdaInternalCommandResult *
+extra_command_close_cnc (GdaConnection *cnc, const gchar **args, GError **error, MainData *data)
 {
 	ConnectionSetting *cs;
 	if (args[0] && *args[0]) {
@@ -1792,9 +1946,8 @@ GdaInternalCommandResult *extra_command_close_cnc (GdaConnection *cnc, const gch
 	return res;
 }
 
-static 
-GdaInternalCommandResult *extra_command_bind_cnc (GdaConnection *cnc, const gchar **args,
-						  GError **error, MainData *data)
+static GdaInternalCommandResult *
+extra_command_bind_cnc (GdaConnection *cnc, const gchar **args, GError **error, MainData *data)
 {
 	ConnectionSetting *cs = NULL;
 	gint i, nargs = g_strv_length ((gchar **) args);
@@ -1898,8 +2051,7 @@ extra_command_quit (GdaConnection *cnc, const gchar **args,
 	return res;
 }
 
-static 
-GdaInternalCommandResult *
+static GdaInternalCommandResult *
 extra_command_cd (GdaConnection *cnc, const gchar **args,
 		  GError **error, MainData *data)
 {
@@ -1958,8 +2110,7 @@ extra_command_cd (GdaConnection *cnc, const gchar **args,
 	return NULL;
 }
 
-static 
-GdaInternalCommandResult *
+static GdaInternalCommandResult *
 extra_command_edit_buffer (GdaConnection *cnc, const gchar **args,
 			   GError **error, MainData *data)
 {
@@ -2051,8 +2202,7 @@ extra_command_edit_buffer (GdaConnection *cnc, const gchar **args,
 	return res;
 }
 
-static 
-GdaInternalCommandResult *
+static GdaInternalCommandResult *
 extra_command_reset_buffer (GdaConnection *cnc, const gchar **args,
 			    GError **error, MainData *data)
 {
@@ -2081,8 +2231,7 @@ extra_command_reset_buffer (GdaConnection *cnc, const gchar **args,
 	return res;
 }
 
-static 
-GdaInternalCommandResult *
+static GdaInternalCommandResult *
 extra_command_show_buffer (GdaConnection *cnc, const gchar **args,
 			   GError **error, MainData *data)
 {
@@ -2097,8 +2246,7 @@ extra_command_show_buffer (GdaConnection *cnc, const gchar **args,
 	return res;
 }
 
-static 
-GdaInternalCommandResult *
+static GdaInternalCommandResult *
 extra_command_exec_buffer (GdaConnection *cnc, const gchar **args,
 			   GError **error, MainData *data)
 {
@@ -2116,8 +2264,7 @@ extra_command_exec_buffer (GdaConnection *cnc, const gchar **args,
 	return res;
 }
 
-static 
-GdaInternalCommandResult *
+static GdaInternalCommandResult *
 extra_command_write_buffer (GdaConnection *cnc, const gchar **args,
 			    GError **error, MainData *data)
 {
@@ -2177,7 +2324,7 @@ extra_command_query_buffer_to_dict (GdaConnection *cnc, const gchar **args,
 		else {
 			gint i;
 			for (i = 0; ; i++) {
-				qname = g_strdup_printf (_("saved_stmt_%d"), i);
+				qname = g_strdup_printf ("saved_stmt_%d", i);
 				stmt = find_statement_in_connection_meta_store (data->current->cnc, qname);
 				if (!stmt)
 					break;
@@ -2475,3 +2622,42 @@ args_as_string_set (const gchar *str)
 {
 	return g_strsplit (str, " ", 3);
 }
+
+
+
+static char **
+completion_func (const char *text, int start, int end)
+{
+#ifdef HAVE_READLINE
+	ConnectionSetting *cs = main_data->current;
+	char **array;
+	gchar **gda_compl;
+	gint i, nb_compl;
+
+	if (!cs)
+		return NULL;
+	gda_compl = gda_completion_list_get (cs->cnc, rl_line_buffer, start, end);
+	if (!gda_compl)
+		return NULL;
+
+	for (nb_compl = 0; gda_compl[nb_compl]; nb_compl++);
+
+	array = malloc (sizeof (char*) * (nb_compl + 1));
+	for (i = 0; i < nb_compl; i++) {
+		char *str;
+		gchar *gstr;
+		gint l;
+
+		gstr = gda_compl[i];
+		l = strlen (gstr) + 1;
+		str = malloc (sizeof (char) * l);
+		memcpy (str, gstr, l);
+		array[i] = str;
+	}
+	array[i] = NULL;
+	g_strfreev (gda_compl);
+	return array;
+#else
+	return NULL;
+#endif
+}
